import sys

# Utils to compile .py to .pyc inside zip file
# XXX: this is implementation detail of .pyc, copied from py_compile.py.
# Unfortunately, there is no way that I know of to write the bytecode into a
# string to be used by ZipFile (using compiler is way too slow). Also, the
# py_compile code has not changed much for 10 years.
# XXX: the code has changed quite a few times in python 3.x timeline, we need
# to keep too many copies. Maybe it is not worth it to support this feature
# altogether ?
from py_compile \
    import \
        PyCompileError
if sys.version_info[0] < 3:
    from _bytecode_2 \
        import \
            bcompile
else:
    from bento.private._bytecode_3 \
        import \
            bcompile



NAME = "bento"
VERSION = "0.1.1"
GIT_REVISION = "d943984a26e54d334e72d22549bd71eab44e1735"

SUMMARY = "A tool to nicely packages your python sotftware"

DESCRIPTION = """Bento is an alternative to distutils-based packaging tools such as distutils,
setuptools or distribute. Bento focus on reproducibility, extensibility and
simplicity (in that order).

Packaging is as simple as writing a bento.info file with a file which looks as
follows::

    Name: Foo
    Author: John Doe

    Library:
        Packages: foo

The package is then installed with bentomaker, the command line interface to
bento::

    bentomaker install

Installing
==========

To install bento, you can either:

    * install bento from itself (recommended)::

        python bootstrap.py
        ./bentomaker install

    * install bento using setuptools (not recommended)::

        python setup.py install

Python3 support
---------------

Bento supports python 3 as is, so there is no need to run 2to3 on it (doing so
will probably break it).

Why you should use bento ?
==========================

    * Straightfoward package description, in an indentation-based syntax
      similar to python
    * Simple packages can have their setup.py automatically converted through
      the 'convert' command
    * Distutils compatibility mode so that a bento package can be installed
      through pip
    * Adding new commands is simple
    * Pluggable build-backend: you can build your C extensions with a real
      build system such as waf or scons.
    * Easy to customize install paths from the command line, with sensible
      defaults on every platform
    * Installing data files such as manpages, configuration, etc... is
      straightforward and customizable through the command line
    * Supports all python versions >= 2.4 (including 3.x)
    * Designed with reproducibility in mind: re-running the same command twice
      should produce the same result (idempotency)
    * Preliminary support for windows installers (.exe), eggs and mpkg.

But bento does more:

    * Designed as a library from the ground up, with a focus on robustness and
      extensibility:

        * new commands can be inserted before/after an existing one without
          modifying the latter (no monkey-patching needed)
        * easy to add command line options to existing commands
        * each command has a pre/post hook
        * API designed such as commands need to know very little from each other.
        * Moving toward a node-based architecture for robust file location
          (waf-based design)
        * No global variable/singleton in bento itself

    * Easily bundable, one-file distribution to avoid extra-dependencies when
      using bento. You only need to add one file to your sources, no need for
      your users to install anything.
    * Basic support for console scripts ala setuptools
    * Dependency-based extension builders (source content change is
      automatically rebuilt)
    * Parallel build support for C extensions
    * Low-level interface to the included build tool to override/change any
      compilation parameter (compilation flag, compiler, etc...)

Planned features:

    * Reliable and fast (parallel) 2->3 convertion.
    * Support for msi packages
    * Reliable conversion between packaging formats on the platforms where it
      makes sense (egg <-> wininst, mpkg <-> egg, etc...)
    * Provide API to enable Linux distributors to write simple extensions for
      packaging bento-packages as they see fit
    * Infrastructure for a correctly designed package index, using
      well-known packaging practices instead of the broken easy_install + pypi
      model (easy mirroring, enforced metadata, indexing to enable
      querying-before-installing, reliable install, etc...).

Bento discussions happen on the bento Mailing list (bento@librelist.com,
archive on `bento-ml`_), and development is on `github`_. Bugs should be
reported on bento `issue-tracker`_. Online `documentation`_ is available on
github as well.

WHILE BENTO IS ALREADY USABLE, IT MAY STILL SIGNIFICANTLY CHANGE IN BACKWARD
INCOMPATIBLE WAYS UNTIL THE FIRST ALPHA.

.. _github: http://github.com/cournape/bento.git
.. _issue-tracker: http://github.com/cournape/bento/issues
.. _documentation: http://cournape.github.com/bento
.. _bento-ml: http://librelist.com/browser/bento
"""

LICENSE = "BSD"

AUTHOR = "David Cournapeau"
AUTHOR_EMAIL = "cournape@gmail.com"

URL = "http://github.com/cournape/Bento"

PLATFORMS = ['any']
KEYWORDS = ['packaging']



def detect(ctx):
    env = ctx.env

    ctx.env["CC"] = ["cc"]
    ctx.env["CC_TGT_F"] = ["-c", "-o"]
    ctx.env["CC_SRC_F"] = []
    ctx.env["CFLAGS"] = []
    ctx.env["CPPPATH"] = []
    ctx.env["CPPPATH_FMT"] = "-I%s"
    ctx.env["LINK"] = ["cc"]
    ctx.env["LINKFLAGS"] = []
    ctx.env["LIBS"] = []
    ctx.env["LIB_FMT"] = "-l%s"
    ctx.env["LIBDIR_FMT"] = "-L%s"



import sys
import os
import copy

import yaku.tools

from yaku.task \
    import \
        task_factory
from yaku.task_manager \
    import \
        extension, CompiledTaskGen, set_extension_hook
from yaku.utils \
    import \
        find_deps, ensure_dir
from yaku.compiled_fun \
    import \
        compile_fun
from yaku.errors \
    import \
        TaskRunFailure
from yaku.scheduler \
    import \
        run_tasks
from yaku.conf \
    import \
        with_conf_blddir
from yaku._config \
    import \
        _OUTPUT
import yaku.tools

ccompile, cc_vars = compile_fun("cc", "${CC} ${CFLAGS} ${APP_DEFINES} ${INCPATH} ${CC_TGT_F}${TGT[0].abspath()} ${CC_SRC_F}${SRC}", False)

shccompile, sgcc_vars = compile_fun("cc", "${CC} ${CFLAGS} ${CFLAGS_SH} ${APP_DEFINES} ${INCPATH} ${CC_TGT_F}${TGT[0].abspath()} ${CC_SRC_F}${SRC}", False)

ccprogram, ccprogram_vars = compile_fun("ccprogram", "${LINK} ${LINK_TGT_F}${TGT[0].abspath()} ${LINK_SRC_F}${SRC} ${APP_LIBDIR} ${APP_LIBS} ${LINKFLAGS}", False)

cshlink, cshlink_vars = compile_fun("cshlib", "${SHLINK} ${APP_LIBDIR} ${APP_LIBS} ${SHLINK_TGT_F}${TGT[0].abspath()} ${SHLINK_SRC_F}${SRC} ${SHLINKFLAGS}", False)

clink, clink_vars = compile_fun("clib", "${STLINK} ${STLINKFLAGS} ${STLINK_TGT_F}${TGT[0].abspath()} ${STLINK_SRC_F}${SRC}", False)

@extension('.c')
def c_hook(self, node):
    tasks = ccompile_task(self, node)
    self.object_tasks.extend(tasks)
    return tasks

def ccompile_task(self, node):
    base = self.env["CC_OBJECT_FMT"] % node.name
    target = node.parent.declare(base)
    ensure_dir(target.abspath())

    task = task_factory("cc")(inputs=[node], outputs=[target], func=ccompile, env=self.env)
    task.gen = self
    task.env_vars = cc_vars
    return [task]

def shared_c_hook(self, node):
    tasks = shared_ccompile_task(self, node)
    self.object_tasks.extend(tasks)
    return tasks

def shared_ccompile_task(self, node):
    base = self.env["CC_OBJECT_FMT"] % node.name
    target = node.parent.declare(base)
    ensure_dir(target.abspath())

    task = task_factory("shcc")(inputs=[node], outputs=[target], func=shccompile, env=self.env)
    task.gen = self
    task.env_vars = cc_vars
    return [task]

def shlink_task(self, name):
    objects = [tsk.outputs[0] for tsk in self.object_tasks]

    folder, base = os.path.split(name)
    tmp = folder + os.path.sep + self.env["SHAREDLIB_FMT"] % base
    target = self.bld.path.declare(tmp)
    ensure_dir(target.abspath())

    task = task_factory("cc_shlink")(inputs=objects, outputs=[target], func=cshlink, env=self.env)
    task.gen = self
    task.env_vars = cshlink_vars
    return [task]

def static_link_task(self, name):
    objects = [tsk.outputs[0] for tsk in self.object_tasks]

    folder, base = os.path.split(name)
    tmp = folder + os.path.sep + self.env["STATICLIB_FMT"] % base
    target = self.bld.path.declare(tmp)
    ensure_dir(target.abspath())

    task = task_factory("cc_stlink")(inputs=objects, outputs=[target], func=clink, env=self.env)
    task.gen = self
    task.env_vars = clink_vars
    return [task]

def ccprogram_task(self, name):
    objects = [tsk.outputs[0] for tsk in self.object_tasks]
    def declare_target():
        folder, base = os.path.split(name)
        tmp = folder + os.path.sep + self.env["PROGRAM_FMT"] % base
        return self.bld.path.declare(tmp)
    target = declare_target()
    ensure_dir(target.abspath())

    task = task_factory("cc_program")(inputs=objects, outputs=[target], func=ccprogram, env=self.env)
    task.gen = self
    task.env_vars = ccprogram_vars
    return [task]

def apply_define(task_gen):
    defines = task_gen.env["DEFINES"]
    task_gen.env["APP_DEFINES"] = [task_gen.env["DEFINES_FMT"] % p for p in defines]

def apply_cpppath(task_gen):
    cpppaths = task_gen.env["CPPPATH"]
    implicit_paths = set([s.parent.srcpath() \
                          for s in task_gen.sources])
    srcnode = task_gen.sources[0].ctx.srcnode

    relcpppaths = []
    for p in cpppaths:
        if not os.path.isabs(p):
            node = srcnode.find_node(p)
            assert node is not None, "could not find %s" % p
            relcpppaths.append(node.bldpath())
        else:
            relcpppaths.append(p)
    cpppaths = list(implicit_paths) + relcpppaths
    task_gen.env["INCPATH"] = [
            task_gen.env["CPPPATH_FMT"] % p
            for p in cpppaths]

def apply_libs(task_gen):
    libs = task_gen.env["LIBS"]
    task_gen.env["APP_LIBS"] = [
            task_gen.env["LIB_FMT"] % lib for lib in libs]

def apply_libdir(task_gen):
    libdir = task_gen.env["LIBDIR"]
    task_gen.env["APP_LIBDIR"] = [
            task_gen.env["LIBDIR_FMT"] % d for d in libdir]

class CCBuilder(yaku.tools.Builder):
    def clone(self):
        return CCBuilder(self.ctx)

    def __init__(self, ctx):
        yaku.tools.Builder.__init__(self, ctx)

    def _compile(self, task_gen, name):
        apply_define(task_gen)
        apply_cpppath(task_gen)

        tasks = task_gen.process()
        for t in tasks:
            t.env = task_gen.env
        return tasks

    def compile(self, name, sources, env=None):
        sources = self.to_nodes(sources)
        task_gen = CompiledTaskGen("cccompile", self.ctx, sources, name)
        task_gen.env = yaku.tools._merge_env(self.env, env)
        tasks = self._compile(task_gen, name)
        self.ctx.tasks.extend(tasks)

        outputs = []
        for t in tasks:
            outputs.extend(t.outputs)
        return outputs

    def try_compile(self, name, body, headers=None):
        return with_conf_blddir(self.ctx, name, body,
                                lambda : yaku.tools.try_task_maker(self.ctx, self._compile, name, body, headers))

    def try_compile_no_blddir(self, name, body, headers=None, env=None):
        return yaku.tools.try_task_maker(self.ctx, self._compile, name, body, headers, env)

    def static_library(self, name, sources, env=None):
        sources = self.to_nodes(sources)
        task_gen = CompiledTaskGen("ccstaticlib", self.ctx, sources, name)
        task_gen.env = yaku.tools._merge_env(self.env, env)

        tasks = self._static_library(task_gen, name)
        self.ctx.tasks.extend(tasks)
        outputs = []
        for t in task_gen.link_task:
            outputs.extend(t.outputs)
        return outputs

    def _static_library(self, task_gen, name):
        apply_define(task_gen)
        apply_cpppath(task_gen)
        apply_libdir(task_gen)
        apply_libs(task_gen)

        tasks = task_gen.process()
        ltask = static_link_task(task_gen, name)
        tasks.extend(ltask)
        for t in tasks:
            t.env = task_gen.env
        task_gen.link_task = ltask
        return tasks

    def try_static_library(self, name, body, headers=None):
        return with_conf_blddir(self.ctx, name, body,
                                lambda : yaku.tools.try_task_maker(self.ctx, self._static_library, name, body, headers))

    def try_static_library_no_blddir(self, name, body, headers=None, env=None):
        return yaku.tools.try_task_maker(self.ctx, self._static_library, name, body, headers, env)

    def shared_library(self, name, sources, env=None):
        sources = self.to_nodes(sources)
        task_gen = CompiledTaskGen("ccsharedlib", self.ctx, sources, name)
        task_gen.env = yaku.tools._merge_env(self.env, env)

        tasks = self._shared_library(task_gen, name)
        self.ctx.tasks.extend(tasks)
        outputs = []
        for t in task_gen.link_task:
            outputs.extend(t.outputs)
        return outputs

    def _shared_library(self, task_gen, name):
        old_hook = set_extension_hook(".c", shared_c_hook)

        apply_define(task_gen)
        apply_cpppath(task_gen)
        apply_libdir(task_gen)
        apply_libs(task_gen)

        tasks = task_gen.process()
        ltask = shlink_task(task_gen, name)
        tasks.extend(ltask)
        for t in tasks:
            t.env = task_gen.env
        task_gen.link_task = ltask

        set_extension_hook(".c", old_hook)
        return tasks

    def try_shared_library(self, name, body, headers=None):
        return with_conf_blddir(self.ctx, name, body,
                                lambda : yaku.tools.try_task_maker(self.ctx, self._shared_library, name, body, headers))

    def try_shared_library_no_blddir(self, name, body, headers=None, env=None):
        return yaku.tools.try_task_maker(self.ctx, self._shared_library, name, body, headers, env)

    def program(self, name, sources, env=None):
        sources = self.to_nodes(sources)
        task_gen = CompiledTaskGen("ccprogram", self.ctx,
                                   sources, name)
        task_gen.env = yaku.tools._merge_env(self.env, env)
        tasks = self._program(task_gen, name)

        self.ctx.tasks.extend(tasks)
        outputs = []
        for t in task_gen.link_task:
            outputs.extend(t.outputs)
        return outputs

    def _program(self, task_gen, name):
        apply_define(task_gen)
        apply_cpppath(task_gen)
        apply_libdir(task_gen)
        apply_libs(task_gen)

        tasks = task_gen.process()
        ltask = ccprogram_task(task_gen, name)
        tasks.extend(ltask)
        for t in tasks:
            t.env = task_gen.env
        task_gen.link_task = ltask
        return tasks

    def try_program(self, name, body, headers=None, env=None):
        return with_conf_blddir(self.ctx, name, body,
                                lambda : yaku.tools.try_task_maker(self.ctx, self._program, name, body, headers, env))

    def try_program_no_blddir(self, name, body, headers=None, env=None):
        return yaku.tools.try_task_maker(self.ctx, self._program, name, body, headers, env)

    def configure(self, candidates=None):
        ctx = self.ctx
        if candidates is None:
            if sys.platform == "win32":
                candidates = ["msvc", "gcc"]
            else:
                candidates = ["gcc", "cc"]

        def _detect_cc():
            detected = None
            sys.path.insert(0, os.path.dirname(yaku.tools.__file__))
            try:
                for cc_type in candidates:
                    _OUTPUT.write("Looking for %s (c compiler) ... " % cc_type)
                    try:
                        mod = __import__(cc_type)
                        if mod.detect(ctx):
                            _OUTPUT.write("yes\n")
                            ctx.env["cc_type"] = cc_type
                            detected = cc_type
                            break
                    except:
                        pass
                    _OUTPUT.write("no!\n")
                return detected
            finally:
                sys.path.pop(0)

        cc_type = _detect_cc()
        if cc_type is None:
            raise ValueError("No C compiler found!")
        cc = ctx.load_tool(cc_type)
        cc.setup(ctx)

        if sys.platform != "win32":
            ar = ctx.load_tool("ar")
            ar.setup(ctx)

        ctx.start_message("Checking whether %s can build objects" % cc_type)
        if self.try_compile("foo", "int foo() {return 0;}"):
            ctx.end_message("yes")
        else:
            ctx.end_message("no")
            ctx.fail_configuration("")
        ctx.start_message("Checking whether %s can build programs" % cc_type)
        if self.try_program("foo", "int main() {return 0;}"):
            ctx.end_message("yes")
        else:
            ctx.end_message("no")
            ctx.fail_configuration("")
        ctx.start_message("Checking whether %s can build static libraries" % cc_type)
        if self.try_static_library("foo", "int foo() {return 0;}"):
            ctx.end_message("yes")
        else:
            ctx.end_message("no")
            ctx.fail_configuration("")
        ctx.start_message("Checking whether %s can link static libraries to exe" % cc_type)
        def f():
            assert self.try_static_library_no_blddir("foo", "int foo() { return 0;}")
            if self.try_program_no_blddir("exe", "int foo(); int main() { return foo();}",
                                          env={"LIBS": ["foo"]}):
                ctx.end_message("yes")
            else:
                ctx.end_message("no")
                ctx.fail_configuration("")
        with_conf_blddir(self.ctx, "exelib", "checking static link", f)

        shared_code = """\
#ifdef _MSC_VER
#define __YAKU_DLL_MARK __declspec(dllexport)
#else
#define __YAKU_DLL_MARK
#endif

__YAKU_DLL_MARK int foo()
{
    return 0;
}
"""
        ctx.start_message("Checking whether %s can build shared libraries" % cc_type)
        if self.try_shared_library("foo", shared_code):
            ctx.end_message("yes")
        else:
            ctx.end_message("no")
            ctx.fail_configuration("")

        ctx.start_message("Checking whether %s can link shared libraries to exe" % cc_type)
        def f():
            assert self.try_shared_library_no_blddir("foo", shared_code)
            if self.try_program_no_blddir("exe", "int foo(); int main() { return foo();}",
                                          env={"LIBS": ["foo"]}):
                ctx.end_message("yes")
            else:
                ctx.end_message("no")
                ctx.fail_configuration("")
        with_conf_blddir(self.ctx, "exeshlib", "checking shared link", f)
        self.configured = True

def get_builder(ctx):
    return CCBuilder(ctx)



"""
Bento, a pythonic packaging solution for python software.

Bento is a packaging solution which aims at being simple and extensible, using
as little magic as possible. Packages are described in a bento.info file which
has a straightforward syntax, and the packaging is driven through bentomaker,
the command line interfance to bento. Sane API are provided so that people can
build their own deployment facilities on top of it.

The code is currently organized as follows:
    - bento.core: core facilities to handle package representation
    - bento.commands: commands as provided by bentomaker
    - bento.compat: compatibility code to provide consistent API to all
    - bento.parser: ply-based lexer/parser for the bento.info format
    - bento.private: bundled packages
"""
import sys
import os

import os.path as op

from bento._config \
    import \
        USE_PRIVATE_MODULES

# FIXME: there has to be a better way to do this ?
for bundled_pkg in ["_ply", "_simplejson", "_yaku", "_six"]:
    v = "BENTO_UNBUNDLE%s" % bundled_pkg.upper()
    if USE_PRIVATE_MODULES and not os.environ.get(v, False):
        m_path = op.join(op.dirname(__file__), "private", bundled_pkg)
        # XXX: we always add bundled packages for now because checking for file
        # existence is too naive (does not work for zip-import)
        sys.path.insert(0, m_path)

try:
    import __package_info
except ImportError:
    __version__ = 'nobuilt'
    __git_revision__ = 'nobuilt'
else:
    from __package_info import VERSION as __version__, GIT_REVISION as __git_revision__



import os
import shutil
import tempfile

import os.path as op

from six.moves \
    import \
        StringIO

import mock

from bento.pypi.register_utils \
    import \
        build_post_data, encode_multipart, post_to_server, DEFAULT_REALM, \
        PyPIConfig, parse_pypirc
from bento.compat.api.moves \
    import \
        unittest
from bento.core \
    import \
        PackageDescription

from six \
    import \
        PY3

import six

if PY3:
    from urllib.request \
        import \
            HTTPPasswordMgr, urlparse, HTTPError, URLError
    _OPENER_DIRECTOR = "urllib.request.OpenerDirector"
else:
    from urllib2 \
        import \
            Request, HTTPPasswordMgr, HTTPError, URLError
    from urlparse \
        import \
            urlparse
    _OPENER_DIRECTOR = "urllib2.OpenerDirector"

class TestRegisterUtils(unittest.TestCase):
    def test_build_post_data(self):
        r_content = six.b("""----------------GHSKFJDLGDS7543FJKLFHRE75642756743254\r\n""" \
"""Content-Disposition: form-data; name="maintainer"\r\n\r\n\r\n""" \
"""----------------GHSKFJDLGDS7543FJKLFHRE75642756743254\r\n""" \
"""Content-Disposition: form-data; name="name"\r\n\r\n""" \
"""foo\r\n""" \
"""----------------GHSKFJDLGDS7543FJKLFHRE75642756743254\r\n""" \
"""Content-Disposition: form-data; name="license"\r\n\r\n\r\n""" \
"""----------------GHSKFJDLGDS7543FJKLFHRE75642756743254\r\n""" \
"""Content-Disposition: form-data; name="author"\r\n\r\n""" \
"""John Doe\r\n""" \
"""----------------GHSKFJDLGDS7543FJKLFHRE75642756743254\r\n""" \
"""Content-Disposition: form-data; name="url"\r\n\r\n\r\n""" \
"""----------------GHSKFJDLGDS7543FJKLFHRE75642756743254\r\n""" \
"""Content-Disposition: form-data; name=":action"\r\n\r\n""" \
"""submit\r\n""" \
"""----------------GHSKFJDLGDS7543FJKLFHRE75642756743254\r\n""" \
"""Content-Disposition: form-data; name="download_url"\r\n\r\n\r\n""" \
"""----------------GHSKFJDLGDS7543FJKLFHRE75642756743254\r\n""" \
"""Content-Disposition: form-data; name="maintainer_email"\r\n\r\n\r\n""" \
"""----------------GHSKFJDLGDS7543FJKLFHRE75642756743254\r\n""" \
"""Content-Disposition: form-data; name="author_email"\r\n\r\n\r\n""" \
"""----------------GHSKFJDLGDS7543FJKLFHRE75642756743254\r\n""" \
"""Content-Disposition: form-data; name="version"\r\n\r\n""" \
"""1.0\r\n""" \
"""----------------GHSKFJDLGDS7543FJKLFHRE75642756743254\r\n""" \
"""Content-Disposition: form-data; name="long_description"\r\n\r\n\r\n""" \
"""----------------GHSKFJDLGDS7543FJKLFHRE75642756743254\r\n""" \
"""Content-Disposition: form-data; name="description"\r\n\r\n\r\n""" \
"""----------------GHSKFJDLGDS7543FJKLFHRE75642756743254--\r\n""" \
"""""")
        bento_info = """\
Name: foo
Version: 1.0
Author: John Doe
"""
        package = PackageDescription.from_string(bento_info)
        post_data = build_post_data(package, "submit")
        content_type, body = encode_multipart(post_data.items(), [])
        self.assertEqual(r_content, body)

    @mock.patch(_OPENER_DIRECTOR, mock.MagicMock())
    def test_register_server(self):
        package = PackageDescription(name="foo")
        repository = 'http://testpypi.python.org/pypi'
        realm = DEFAULT_REALM
        config = PyPIConfig(username="cdavid", password="yoyo", repository=repository, realm=realm)

        auth = HTTPPasswordMgr()
        host = urlparse(config.repository)[0]
        auth.add_password(config.realm, host, config.username, config.password)

        post_data = build_post_data(package, "submit")
        code, msg = post_to_server(post_data, config, auth)
        self.assertEqual(code, 200)
        self.assertEqual(msg, "OK")

    @mock.patch("%s.open" % _OPENER_DIRECTOR,
                mock.MagicMock(side_effect=HTTPError("", 404, "", {}, None)))
    def test_register_server_http_errors(self):
        code, msg = self._test_register_server_errors()
        self.assertEqual(code, 404)
        self.assertEqual(msg, "")

    @mock.patch("%s.open" % _OPENER_DIRECTOR,
                mock.MagicMock(side_effect=URLError("")))
    def test_register_server_url_errors(self):
        code, msg = self._test_register_server_errors()
        self.assertEqual(code, 500)

    def _test_register_server_errors(self):
        package = PackageDescription(name="foo")
        config = PyPIConfig.from_string("""
[distutils]
index-servers = pypi

[pypi]
username = cdavid
password = yoyo
server = http://testpypi.python.org
""")

        post_data = build_post_data(package, "submit")
        return post_to_server(post_data, config)

class TestReadPyPI(unittest.TestCase):
    def test_pypi_config(self):
        data = """\
[distutils]
index-servers =
    test

[test]
username:cdavid
password:yoyo
repository:http://testpypi.python.org
"""
        config = PyPIConfig.from_file(StringIO(data), repository="test")
        self.assertEqual(config.username, "cdavid")
        self.assertEqual(config.password, "yoyo")

    def test_new_format_default(self):
        data = """\
[distutils]
index-servers =

[pypi]
username:cdavid
password:yoyo
repository:http://testpypi.python.org
"""
        config = parse_pypirc(StringIO(data), 'pypi')
        self.assertEqual(config.username, "cdavid")
        self.assertEqual(config.password, "yoyo")
        self.assertEqual(config.repository, "http://testpypi.python.org")

    def test_invalid_format(self):
        self.assertRaises(ValueError, parse_pypirc, StringIO(""))

    def test_new_format(self):
        data = """\
[distutils]
index-servers =
    test
    pypi

[test]
username:cdavid
password:yoyo
repository:http://testpypi.python.org

[pypi]
username:david
password:yeye
repository:http://pypi.python.org
"""
        config = parse_pypirc(StringIO(data), "pypi")
        self.assertEqual(config.username, "david")
        self.assertEqual(config.password, "yeye")

        config = parse_pypirc(StringIO(data), repository="test")
        self.assertEqual(config.username, "cdavid")
        self.assertEqual(config.password, "yoyo")
        self.assertEqual(config.repository, "http://testpypi.python.org")

    def test_old_format(self):
        data = """\
[server-login]
username:cdavid
password:yoyo
repository:http://testpypi.python.org
"""
        config = parse_pypirc(StringIO(data))
        self.assertEqual(config.username, "cdavid")
        self.assertEqual(config.password, "yoyo")
        self.assertEqual(config.repository, "http://testpypi.python.org")

class TestReadPyPIDefault(unittest.TestCase):
    def setUp(self):
        self.d = tempfile.mkdtemp()
        try:
            self.p = mock.patch("os.path.expanduser", lambda x: self.d)
            self.p.start()
        except:
            shutil.rmtree(self.d)
            raise

    def tearDown(self):
        self.p.stop()
        shutil.rmtree(self.d)

    def test_read_pypirc(self):
        # Guard to make sure we don't write an actual file by mistake
        try:
            fp = open(op.join(op.expanduser("~"), ".pypirc"), "rt")
            fp.close()
            raise ValueError()
        except IOError:
            pass

        fp = open(op.join(op.expanduser("~"), ".pypirc"), "wt")
        try:
            fp.write("")
        finally:
            fp.close()



from unittest \
    import \
        TestCase

from bento.utils.utils \
    import \
        extract_exception, is_string
from bento.parser.lexer \
    import \
        MyLexer, indent_generator, post_process

def split(s):
    ret = []
    for i in s.split(" "):
        ret.extend(i.splitlines())
    return ret

class TestLexer(TestCase):
    def setUp(self):
        self.lexer = MyLexer()

    def _test(self, data, ref):
        self.lexer.input(data)
        self._test_impl(data, ref)

    def _test_impl(self, data, ref):
        res = []
        while True:
            tok = self.lexer.token()
            if not tok:
                break
            res.append(tok.type)

        if is_string(ref):
            ref = split(ref)
        try:
            self.assertEqual(res, ref)
        except AssertionError:
            e = extract_exception()
            cnt = 0
            for i, j in zip(res, ref):
                if not i == j:
                    break
                cnt += 1
            print("Break at index %d" % cnt)
            raise e

    def _get_tokens(self, data):
        self.lexer.input(data)
        return [t for t in self.lexer.token_stream]

# Test tokenizer stage before indentation generation
class TestLexerStageOne(TestLexer):
    def setUp(self):
        self.lexer = MyLexer(stage="raw")

    def test_single_line(self):
        data = """\
Name: yo
"""
        ref = ["WORD", "COLON", "WS", "WORD", "NEWLINE"]
        self._test(data, ref)

    def test_two_lines(self):
        data = """\
Name: yo
Summary: a brief summary
"""
        ref = ["WORD", "COLON", "WS", "WORD", "NEWLINE",
               "WORD", "COLON", "WS", "WORD", "WS", "WORD", "WS", "WORD", "NEWLINE"]
        self._test(data, ref)

    def test_tab(self):
        data = """\
Library:
\tpackages
"""
        self.assertRaises(SyntaxError, lambda: self._test(data, []))

class TestLexerStageTwo(TestLexer):
    def setUp(self):
        self.lexer = MyLexer(stage="escape_detected")

    def test_simple(self):
        data = "yoyo"
        ref = "WORD"
        self._test(data, ref)

    def test_empty(self):
        data = ""
        ref = []
        self._test(data, ref)

    def test_simple_escape(self):
        data = "yoyo\ "
        ref = "WORD WS"
        self._test(data, ref)

        tokens = self._get_tokens(data)
        self.assertEqual(tokens[0].escaped, False)
        self.assertEqual(tokens[1].escaped, True)

    def test_double_escape(self):
        data = "yoyo\\\\ "
        ref = "WORD BACKSLASH WS"
        self._test(data, ref)

        tokens = self._get_tokens(data)
        self.assertEqual(tokens[0].escaped, False)
        self.assertEqual(tokens[1].escaped, True)
        self.assertEqual(tokens[2].escaped, False)

    def test_wrong_escape(self):
        data = "yoyo\\"
        def f():
            self._get_tokens(data)
        self.assertRaises(SyntaxError, f)

class TestLexerStageThree(TestLexer):
    def setUp(self):
        self.lexer = MyLexer(stage="escape_merged")

    def test_simple(self):
        data = "yoyo"
        ref = "WORD"
        self._test(data, ref)

    def test_empty(self):
        data = ""
        ref = []
        self._test(data, ref)

    def test_simple_escape(self):
        data = "yoyo\ "
        ref = "WORD"
        self._test(data, ref)

        tokens = self._get_tokens(data)
        self.assertEqual(tokens[0].value, "yoyo ")

    def test_simple_escape2(self):
        data = "\ yoyo\ "
        ref = "WORD"
        self._test(data, ref)

        tokens = self._get_tokens(data)
        self.assertEqual(tokens[0].value, " yoyo ")

    def test_double_escape(self):
        data = "yoyo\ \ yaya"
        ref = "WORD"
        self._test(data, ref)

        tokens = self._get_tokens(data)
        self.assertEqual(tokens[0].value, "yoyo  yaya")

    def test_literal_escape(self):
        data = "yoyo\\\\"
        ref = "WORD"
        self._test(data, ref)

        tokens = self._get_tokens(data)
        self.assertEqual(tokens[0].value, "yoyo\\")

class TestLexerStageFour(TestLexer):
    def setUp(self):
        self.lexer = MyLexer(stage="indent_generated")

    def test_single_line(self):
        data = """\
Name: yo
"""
        ref = ["WORD", "COLON", "WS", "WORD", "NEWLINE"]
        self._test(data, ref)

    def test_two_lines(self):
        data = """\
Name: yo
Summary: a brief summary
"""
        ref = ["WORD", "COLON", "WS", "WORD", "NEWLINE",
               "WORD", "COLON", "WS", "WORD", "WS", "WORD", "WS", "WORD", "NEWLINE"]
        self._test(data, ref)

    def test_simple_indent(self):
        data = """\
Packages:
    yo
"""
        ref = """\
WORD COLON NEWLINE
INDENT WORD NEWLINE
DEDENT
"""
        self._test(data, ref)

    def test_simple_indent2(self):
        data = """\
Packages:
    yo,
    yeah
"""
        ref = """\
WORD COLON NEWLINE
INDENT
WORD COMMA NEWLINE
WORD NEWLINE
DEDENT"""
        self._test(data, ref)

    def test_indent_newlines(self):
        data = """\
Description:
    some
    words

    and then more
"""
        ref = """\
WORD COLON NEWLINE
INDENT
WORD NEWLINE
WORD NEWLINE
WORD WS WORD WS WORD NEWLINE
DEDENT
"""
        self._test(data, ref)

    def test_double_indentation(self):
        data = """\
Description:
    some
     words
"""
        ref = """\
WORD COLON NEWLINE
INDENT
WORD NEWLINE
INDENT
WORD NEWLINE
DEDENT DEDENT
"""
        self._test(data, ref)

    def test_simple_dedent(self):
        data = """\
Packages:
    some
Name: words
"""
        ref = """\
WORD COLON NEWLINE
INDENT WORD NEWLINE DEDENT
WORD COLON WS WORD NEWLINE
"""
        self._test(data, ref)

    def test_simple_indent_dedent(self):
        data = """\
Library:
    Packages:
        yo
        yo.foo
    Modules:
        foo.py
    Extension: _bar
"""
        ref = """\
WORD COLON NEWLINE
INDENT
WORD COLON NEWLINE
INDENT
WORD NEWLINE
WORD NEWLINE
DEDENT
WORD COLON NEWLINE
INDENT WORD NEWLINE
DEDENT
WORD COLON WS WORD NEWLINE
DEDENT
"""
        self._test(data, ref)

    def test_complex_indent(self):
        data = """\
Library:
    Packages:
        yo
        yo.foo
    Modules:
        foo.py
        bar.py
Extension: yeah
Extension: yeah2
"""
        ref = """\
WORD COLON NEWLINE
INDENT
WORD COLON NEWLINE
INDENT WORD NEWLINE
WORD NEWLINE
DEDENT
WORD COLON NEWLINE
INDENT WORD NEWLINE
WORD NEWLINE
DEDENT DEDENT
WORD COLON WS WORD NEWLINE
WORD COLON WS WORD NEWLINE
"""
        self._test(data, ref)

    def test_indent_value(self):
        data = """\
Description: some
    words and whatnot
      .
"""
        ref = """\
WORD COLON WS WORD NEWLINE
INDENT WORD WS WORD WS WORD NEWLINE
INDENT WORD NEWLINE
DEDENT DEDENT
"""
        self._test(data, ref)
        tokens = self._get_tokens(data)

        indents = [t for t in tokens if t.type in ["INDENT", "DEDENT"]]
        self.assertEqual(indents[0].value, 4)
        self.assertEqual(indents[1].value, 6)
        self.assertEqual(indents[2].value, 6)
        self.assertEqual(indents[3].value, 4)

    def test_indent_value2(self):
        data = """\
Description: some
    words and whatnot
      .
Name: yo
"""
        ref = """\
WORD COLON WS WORD NEWLINE
INDENT WORD WS WORD WS WORD NEWLINE
INDENT WORD NEWLINE
DEDENT DEDENT
WORD COLON WS WORD NEWLINE
"""
        self._test(data, ref)
        tokens = self._get_tokens(data)

        indents = [t for t in tokens if t.type in ["INDENT", "DEDENT"]]
        self.assertEqual(indents[0].value, 4)
        self.assertEqual(indents[1].value, 6)
        self.assertEqual(indents[2].value, 6)
        self.assertEqual(indents[3].value, 4)

class TestLexerStageFive(TestLexer):
    def setUp(self):
        self.lexer = MyLexer(stage='post_processed')

    def test_single_line(self):
        data = """\
Name: yo
"""
        ref = ["NAME_ID", "COLON", "WORD"]
        self._test(data, ref)

    def test_two_lines(self):
        data = """\
Name: yo
Summary: a brief summary
"""
        ref = ["NAME_ID", "COLON", "WORD",
               "SUMMARY_ID", "COLON", "WS", "WORD", "WS", "WORD", "WS", "WORD"]
        self._test(data, ref)

    def test_simple_indent(self):
        data = """\
Packages:
    yo
"""
        ref = ["PACKAGES_ID", "COLON",
               "INDENT", "WORD", "DEDENT"]
        self._test(data, ref)

    def test_simple_indent2(self):
        data = """\
Packages:
    yo,
    yeah
"""
        ref = ["PACKAGES_ID", "COLON",
               "INDENT",
               "WORD", "COMMA",
               "WORD",
               "DEDENT"]
        self._test(data, ref)

    def test_indent_newlines(self):
        data = """\
Description:
    some
    words

    and then more
"""
        ref = ["DESCRIPTION_ID", "COLON", "NEWLINE",
               "INDENT", "WORD", "NEWLINE",
               "WORD", "NEWLINE",
               "WORD", "WS", "WORD", "WS", "WORD",
               "DEDENT"]
        self._test(data, ref)

    def test_double_indentation(self):
        data = """\
Description:
    some
     words
"""
        ref = ["DESCRIPTION_ID", "COLON", "NEWLINE",
               "INDENT", "WORD", "NEWLINE",
               "INDENT", "WORD",
               "DEDENT", "DEDENT"]
        self._test(data, ref)

    def test_simple_dedent(self):
        data = """\
Packages:
    some
Name: words
"""
        ref = ["PACKAGES_ID", "COLON",
               "INDENT", "WORD", "DEDENT",
               "NAME_ID", "COLON", "WORD"]
        self._test(data, ref)

    def test_simple_indent_dedent(self):
        data = """\
Library:
    Packages:
        yo
        yo.foo
    Modules:
        foo.py
    Extension: _bar
"""
        ref = ["LIBRARY_ID", "COLON",
               "INDENT",
               "PACKAGES_ID", "COLON",
               "INDENT",
               "WORD",
               "WORD",
               "DEDENT",
               "MODULES_ID", "COLON",
               "INDENT", "WORD",
               "DEDENT",
               "EXTENSION_ID", "COLON", "WORD",
               "DEDENT"]
        self._test(data, ref)

    def test_complex_indent(self):
        data = """\
Library:
    Packages:
        yo
        yo.foo
    Modules:
        foo.py
        bar.py
Extension: yeah
Extension: yeah2
"""
        ref = ["LIBRARY_ID", "COLON",
               "INDENT",
               "PACKAGES_ID", "COLON",
               "INDENT", "WORD",
               "WORD",
               "DEDENT",
               "MODULES_ID", "COLON",
               "INDENT", "WORD",
               "WORD",
               "DEDENT", "DEDENT",
               "EXTENSION_ID", "COLON", "WORD",
               "EXTENSION_ID", "COLON", "WORD"]
        self._test(data, ref)

    def test_indent_value(self):
        data = """\
Description: some
    words and whatnot
      .
"""
        ref = ["DESCRIPTION_ID", "COLON", "WS", "WORD", "NEWLINE",
               "INDENT", "WORD", "WS", "WORD", "WS", "WORD", "NEWLINE",
               "INDENT", "WORD",
               "DEDENT", "DEDENT"]
        self._test(data, ref)
        tokens = self._get_tokens(data,)

        indents = [t for t in tokens if t.type in ["INDENT", "DEDENT"]]
        self.assertEqual(indents[0].value, 4)
        self.assertEqual(indents[1].value, 6)
        self.assertEqual(indents[2].value, 6)
        self.assertEqual(indents[3].value, 4)

    def test_indent_value2(self):
        data = """\
Description: some
    words and whatnot
      .
Name: yo
"""
        ref = ["DESCRIPTION_ID", "COLON", "WS", "WORD", "NEWLINE",
               "INDENT", "WORD", "WS", "WORD", "WS", "WORD", "NEWLINE",
               "INDENT", "WORD",
               "DEDENT", "DEDENT",
               "NAME_ID", "COLON", "WORD"]
        self._test(data, ref)
        tokens = self._get_tokens(data,)

        indents = [t for t in tokens if t.type in ["INDENT", "DEDENT"]]
        self.assertEqual(indents[0].value, 4)
        self.assertEqual(indents[1].value, 6)
        self.assertEqual(indents[2].value, 6)
        self.assertEqual(indents[3].value, 4)

    def test_comma(self):
        data = """\
Library:
    Packages:
        yo,
        bar,
        yo
"""
        ref = ["LIBRARY_ID", "COLON",
               "INDENT", "PACKAGES_ID", "COLON",
               "INDENT", "WORD", "COMMA",
               "WORD", "COMMA",
               "WORD",
               "DEDENT",
               "DEDENT"
               ]
        self._test(data, ref)

    def test_comma2(self):
        data = """\
Library:
    Packages: foo, bar
    Packages:
        foo,
        bar
    Packages: foo,
        bar
"""
        ref = ["LIBRARY_ID", "COLON",
               "INDENT",
               "PACKAGES_ID", "COLON", "WORD", "COMMA", "WORD",
               "PACKAGES_ID", "COLON", "INDENT", "WORD", "COMMA", "WORD", "DEDENT",
               "PACKAGES_ID", "COLON", "WORD", "COMMA", "INDENT", "WORD",
               "DEDENT",
               "DEDENT"]
        self._test(data, ref)

    def test_tab(self):
        data = """\
Library:
\tpackages
"""
        self.assertRaises(SyntaxError, lambda: self._test(data, []))

    def test_rest_literal1(self):
        data = '''\
Description:
    Sphinx is a tool that makes it easy to create intelligent and beautiful
    documentation for Python projects (or other documents consisting of
    multiple reStructuredText sources), written by Georg Brandl.
    It was originally created to translate the new Python documentation,
    but has now been cleaned up in the hope that it will be useful to many
    other projects.
'''
        ref_str = """\
DESCRIPTION_ID COLON NEWLINE
INDENT
    WORD WS WORD WS WORD WS WORD WS WORD WS WORD WS WORD WS WORD WS WORD WS
    WORD WS WORD WS WORD WS WORD
NEWLINE
    WORD WS WORD WS WORD WS WORD WS LPAR WORD WS WORD WS WORD WS WORD WS WORD
NEWLINE
    WORD WS WORD WS WORD RPAR COMMA WS WORD WS WORD WS WORD WS WORD
NEWLINE
    WORD WS WORD WS WORD WS WORD WS WORD WS WORD WS WORD WS WORD WS WORD WS
    WORD COMMA
NEWLINE
    WORD WS WORD WS WORD WS WORD WS WORD WS WORD WS WORD WS WORD WS WORD WS
    WORD WS WORD WS WORD WS WORD WS WORD WS WORD WS WORD NEWLINE WORD WS WORD
DEDENT
"""

        self._test(data, split(ref_str))

        data = """
Description:
    Sphinx uses reStructuredText as its markup language, and many of its strengths
    come from the power and straightforwardness of reStructuredText and its
    parsing and translating suite, the Docutils.
"""
        ref_str = """\
DESCRIPTION_ID COLON NEWLINE
INDENT
    WORD WS WORD WS WORD WS WORD WS WORD WS WORD WS WORD COMMA WS WORD WS WORD
    WS WORD WS WORD WS WORD
NEWLINE
    WORD WS WORD WS WORD WS WORD WS WORD WS WORD WS WORD WS WORD WS WORD WS WORD
NEWLINE
    WORD WS WORD WS WORD WS WORD COMMA WS WORD WS WORD
DEDENT
"""
        self._test(data, split(ref_str))
    
        data = """\
Description:
    Although it is still under constant development, the following features
    are already present, work fine and can be seen "in action" in the Python docs:
"""

        ref_str = """\
DESCRIPTION_ID COLON NEWLINE
INDENT
    WORD WS WORD WS WORD WS WORD WS WORD WS WORD WS WORD COMMA WS WORD WS WORD
    WS WORD
NEWLINE
    WORD WS WORD WS WORD COMMA WS WORD WS WORD WS WORD WS WORD WS WORD WS WORD
    WS WORD WS WORD WS WORD WS WORD WS WORD WS WORD COLON
DEDENT
"""

        self._test(data, split(ref_str))

        data = """\
Description:
    * Output formats: HTML (including Windows HTML Help), plain text and LaTeX,
      for printable PDF versions
"""

        ref_str = """\
DESCRIPTION_ID COLON NEWLINE
INDENT
    WORD WS WORD WS WORD COLON WS WORD WS LPAR WORD WS WORD WS WORD WS WORD RPAR
    COMMA WS WORD WS WORD WS WORD WS WORD COMMA
NEWLINE
    INDENT WORD WS WORD WS WORD WS WORD
DEDENT
DEDENT
"""

        self._test(data, split(ref_str))

#    def test_rest_literal2(self):
#        data = '''\
#Description:
#    Sphinx is a tool that makes it easy to create intelligent and beautiful
#    documentation for Python projects (or other documents consisting of
#    multiple reStructuredText sources), written by Georg Brandl.
#    It was originally created to translate the new Python documentation,
#    but has now been cleaned up in the hope that it will be useful to many
#    other projects.
#
#    Sphinx uses reStructuredText as its markup language, and many of its strengths
#    come from the power and straightforwardness of reStructuredText and its
#    parsing and translating suite, the Docutils.
#
#    Although it is still under constant development, the following features
#    are already present, work fine and can be seen "in action" in the Python docs:
#
#    * Output formats: HTML (including Windows HTML Help), plain text and LaTeX,
#      for printable PDF versions
#    * Extensive cross-references: semantic markup and automatic links
#      for functions, classes, glossary terms and similar pieces of information
#    * Hierarchical structure: easy definition of a document tree, with automatic
#      links to siblings, parents and children
#    * Automatic indices: general index as well as a module index
#    * Code handling: automatic highlighting using the Pygments highlighter
#    * Various extensions are available, e.g. for automatic testing of snippets
#      and inclusion of appropriately formatted docstrings.
#
#    A development egg can be found `here
#    <http://bitbucket.org/birkenfeld/sphinx/get/tip.gz#egg=Sphinx-dev>`_.
#'''

    def test_ref_literal2(self):
        # Test transition from SCANING_MULTILINE_FIELD
        data = """\
Description: a summary
Name: yo
"""
        ref_str = """\
DESCRIPTION_ID COLON WS WORD WS WORD
NAME_ID COLON WORD
"""
        self._test(data, split(ref_str))

    def test_indented_multiline(self):
        data = """\
Path: path
    Description: descr
    Name: name
"""

        ref_str = """\
PATH_ID COLON WORD
INDENT
DESCRIPTION_ID COLON WS WORD
NAME_ID COLON WORD
DEDENT
"""
        self._test(data, split(ref_str))

    def test_indented_multiline2(self):
        data = """\
Classifiers:
    foo
Path: path
    Description: descr
    Name: name
"""

        ref_str = """\
CLASSIFIERS_ID COLON
INDENT
WORD
DEDENT
PATH_ID COLON WORD
INDENT
DESCRIPTION_ID COLON WS WORD
NAME_ID COLON WORD
DEDENT
"""
        self._test(data, split(ref_str))

class TestNewLines(TestLexer):
    def setUp(self):
        self.lexer = MyLexer(stage="post_processed")

    # Test we throw away NEWLINES except in literals
    def test_lastnewline(self):
        data = """\
Name: yo
"""
        ref_str = """\
NAME_ID COLON WORD
"""
        self._test(data, split(ref_str))

    def test_start_with_newlines(self):
        data = """\

Name: yo
"""
        ref_str = """\
NAME_ID COLON WORD
"""
        self._test(data, split(ref_str))

    def test_start_with_newlines2(self):
        data = """\
Summary: a summary
"""
        ref_str = """\
SUMMARY_ID COLON WS WORD WS WORD
"""
        self._test(data, split(ref_str))

    def test_dedent_newline(self):
        data = """\
Description: Sphinx
    is
        a
    tool
"""

        ref_str = """\
DESCRIPTION_ID COLON WS WORD NEWLINE
INDENT WORD NEWLINE
INDENT WORD NEWLINE
DEDENT WORD
DEDENT
"""
        self._test(data, split(ref_str))

    def test_single_line(self):
        data = "Name: word"

        ref_str = "NAME_ID COLON WORD"
        self._test(data, split(ref_str))

class TestComment(TestLexer):
    def setUp(self):
        self.lexer = MyLexer(stage="post_processed")

    def test_simple(self):
        data = """\
# Simple comment
Name: foo
"""

        ref_str = "NAME_ID COLON WORD"
        self._test(data, split(ref_str))

    def test_simple_inline(self):
        data = """\
Name: foo # inline comment
"""

        ref_str = "NAME_ID COLON WORD"
        self._test(data, split(ref_str))

    def test_simple_inline2(self):
        data = """\
ExtraSourceFiles:
    # indented comment
    foo
"""

        ref_str = "EXTRA_SOURCE_FILES_ID COLON INDENT WORD DEDENT"
        self._test(data, split(ref_str))



import os
import sys
import shutil
import tempfile

import posixpath

import mock

from bento.compat.api.moves \
    import \
        unittest

from bento.core.options \
    import \
        PackageOptions
from bento.core \
    import \
        PackageDescription
from bento.core.node \
    import \
        create_root_with_source_tree
from bento.commands.wrapper_utils \
    import \
        run_command_in_context
from bento.commands.tests.utils \
    import \
        prepare_configure
from bento.backends.yaku_backend \
    import \
        ConfigureYakuContext
from bento.commands.configure \
    import \
        _compute_scheme, set_scheme_unix, set_scheme_win32

BENTO_INFO = """\
Name: Sphinx
Version: 0.6.3
Summary: Python documentation generator
Url: http://sphinx.pocoo.org/
DownloadUrl: http://pypi.python.org/pypi/Sphinx
Description: Some long description.
Author: Georg Brandl
AuthorEmail: georg@python.org
Maintainer: Georg Brandl
MaintainerEmail: georg@python.org
License: BSD
"""

PY_VERSION_SHORT = ".".join(str(_) for _ in sys.version_info[:2])
PY_VERSION_SHORT_NO_DOT = "".join(str(_) for _ in sys.version_info[:2])

class TestConfigureCommand(unittest.TestCase):
    def setUp(self):
        self.d = tempfile.mkdtemp()
        self.root = create_root_with_source_tree(self.d, os.path.join(self.d, "build"))

        self.old_dir = os.getcwd()
        os.chdir(self.d)

    def tearDown(self):
        os.chdir(self.old_dir)
        shutil.rmtree(self.d)

    def test_simple(self):
        root = self.root
        run_node = root.find_node(self.d)

        conf, configure = prepare_configure(run_node, BENTO_INFO, ConfigureYakuContext)
        run_command_in_context(conf, configure)

    def test_flags(self):
        bento_info = """\
Name: foo

Flag: floupi
    Description: some floupi flag
    Default: true
"""
        run_node = self.root.find_node(self.d)

        conf, configure = prepare_configure(run_node, bento_info, ConfigureYakuContext, ["--floupi=false"])
        run_command_in_context(conf, configure)

UNIX_REFERENCE = {
        'destdir': "/",
        'prefix': None,
        'eprefix': None,
        'bindir': '$eprefix/bin',
        'sbindir': '$eprefix/sbin',
        'libexecdir': '$eprefix/libexec',
        'sysconfdir': '$prefix/etc',
        'sharedstatedir': '$prefix/com',
        'localstatedir': '$prefix/var',
        'libdir': '$eprefix/lib',
        'includedir': '$prefix/include',
        'datarootdir': '$prefix/share',
        'datadir': '$datarootdir',
        'mandir': '$datarootdir/man',
        'infodir': '$datarootdir/info',
        'localedir': '$datarootdir/locale',
        'docdir': '$datarootdir/doc/$pkgname',
        'htmldir': '$docdir',
        'dvidir': '$docdir',
        'psdir': '$docdir',
        'pdfdir': '$docdir',
        'sitedir': '$libdir/python$py_version_short/site-packages',
        'pkgdatadir': '$datadir/$pkgname'
}

WIN32_REFERENCE = {
        'destdir': "C:\\",
        'prefix': None,
        'eprefix': r'$prefix',
        'bindir': r'$eprefix\Scripts',
        'sbindir': r'$eprefix\Scripts',
        'libexecdir': r'$eprefix\Scripts',
        'sysconfdir': r'$prefix\etc',
        'sharedstatedir': r'$prefix\com',
        'localstatedir': r'$prefix\var',
        'libdir': r'$eprefix\lib',
        'includedir': r'$prefix\include',
        'datarootdir': r'$prefix\share',
        'datadir': r'$datarootdir',
        'mandir': r'$datarootdir\man',
        'infodir': r'$datarootdir\info',
        'localedir': r'$datarootdir\locale',
        'docdir': r'$datarootdir\doc\$pkgname',
        'htmldir': r'$docdir',
        'dvidir': r'$docdir',
        'psdir': r'$docdir',
        'pdfdir': r'$docdir',
        'sitedir': r'$prefix\Lib\site-packages',
        'pkgdatadir': r'$datadir\$pkgname'
    }
MOCK_DEBIAN_SCHEME = {
        'purelib': '$base/local/lib/python$py_version_short/dist-packages',
        'headers': '$base/local/include/python$py_version_short/$dist_name',
        'scripts': '$base/local/bin',
}


class TestUnixScheme(unittest.TestCase):
    def setUp(self):
        super(TestUnixScheme, self).setUp()
        options = mock.Mock()
        options.eprefix = None
        options.prefix = None

        self.options = options

    def _compute_scheme(self, bento_info, options):
        package_options = PackageOptions.from_string(bento_info)
        pkg = PackageDescription.from_string(bento_info)
        scheme = _compute_scheme(package_options)
        set_scheme_unix(scheme, options, pkg)

        return scheme


    @mock.patch("sys.platform", "linux2")
    @mock.patch("bento.commands.configure.virtualenv_prefix", lambda: None)
    @mock.patch("bento.core.platforms.sysconfig.bento.utils.path.find_root", lambda ignored: r"/")
    @mock.patch("distutils.command.install.INSTALL_SCHEMES", {"unix_prefix": UNIX_REFERENCE})
    def test_scheme_default(self):
        bento_info = """\
Name: foo
"""
        self.options.prefix = self.eprefix = None

        scheme = self._compute_scheme(bento_info, self.options)
        prefix = scheme.pop("prefix")
        eprefix = scheme.pop("eprefix")
        py_version_short = scheme.pop("py_version_short")
        pkgname = scheme.pop("pkgname")

        self.assertEqual(prefix, "/usr/local")
        self.assertEqual(eprefix, "/usr/local")
        self.assertEqual(pkgname, "foo")
        self.assertEqual(py_version_short, PY_VERSION_SHORT)

        # Check that other values in scheme have not been modified
        for k, v in scheme.items():
            self.assertEqual(UNIX_REFERENCE[k], v, "discrepency for path %s: %s vs %s" % (k, UNIX_REFERENCE[k], v))

    @mock.patch("sys.platform", "darwin")
    @mock.patch("bento.core.platforms.sysconfig.bento.utils.path.find_root", lambda ignored: r"/")
    @mock.patch("bento.commands.configure.op.normpath", posixpath.normpath)
    @mock.patch("bento.commands.configure.virtualenv_prefix", lambda: None)
    @mock.patch("sys.prefix", "/Library/Frameworks/Python.framework/Versions/2.8")
    @mock.patch("sys.exec_prefix", "/Exec/Library/Frameworks/Python.framework/Versions/2.8")
    def test_scheme_default_darwin(self):
        bento_info = """\
Name: foo
"""
        self.options.prefix = self.eprefix = None

        scheme = self._compute_scheme(bento_info, self.options)
        prefix = scheme.pop("prefix")
        eprefix = scheme.pop("eprefix")
        py_version_short = scheme.pop("py_version_short")
        pkgname = scheme.pop("pkgname")

        self.assertEqual(prefix, sys.prefix)
        self.assertEqual(eprefix, sys.exec_prefix)
        self.assertEqual(pkgname, "foo")
        self.assertEqual(py_version_short, PY_VERSION_SHORT)

        # Check that other values in scheme have not been modified
        for k, v in scheme.items():
            self.assertEqual(UNIX_REFERENCE[k], v)

    @mock.patch("sys.platform", "linux2")
    @mock.patch("bento.core.platforms.sysconfig.bento.utils.path.find_root", lambda ignored: r"/")
    def test_scheme_with_prefix(self):
        bento_info = """\
Name: foo
"""
        self.options.prefix = "/home/guido"

        scheme = self._compute_scheme(bento_info, self.options)
        prefix = scheme.pop("prefix")
        eprefix = scheme.pop("eprefix")
        py_version_short = scheme.pop("py_version_short")
        pkgname = scheme.pop("pkgname")

        self.assertEqual(prefix, "/home/guido")
        self.assertEqual(eprefix, "/home/guido")
        self.assertEqual(pkgname, "foo")
        self.assertEqual(py_version_short, PY_VERSION_SHORT)

        # Check that other values in scheme have not been modified
        for k, v in scheme.items():
            self.assertEqual(UNIX_REFERENCE[k], v)

        self.options.eprefix = "/home/exec/guido"

        scheme = self._compute_scheme(bento_info, self.options)
        prefix = scheme.pop("prefix")
        eprefix = scheme.pop("eprefix")
        py_version_short = scheme.pop("py_version_short")
        pkgname = scheme.pop("pkgname")

        self.assertEqual(prefix, "/home/guido")
        self.assertEqual(eprefix, "/home/exec/guido")
        self.assertEqual(pkgname, "foo")
        self.assertEqual(py_version_short, PY_VERSION_SHORT)

        # Check that other values in scheme have not been modified
        for k, v in scheme.items():
            self.assertEqual(UNIX_REFERENCE[k], v)

    @mock.patch("sys.platform", "linux2")
    def test_scheme_with_eprefix_fail(self):
        bento_info = """\
Name: foo
"""
        self.options.eprefix = "/home/guido"

        self.assertRaises(NotImplementedError, lambda: self._compute_scheme(bento_info, self.options))

    @mock.patch("sys.platform", "linux2")
    @mock.patch("bento.commands.configure.virtualenv_prefix", lambda: None)
    @mock.patch("bento.core.platforms.sysconfig.bento.utils.path.find_root", lambda ignored: r"/")
    @mock.patch("distutils.command.install.INSTALL_SCHEMES", {"unix_local": MOCK_DEBIAN_SCHEME}, create=True)
    def test_scheme_debian(self):
        bento_info = """\
Name: foo
"""

        scheme = self._compute_scheme(bento_info, self.options)
        prefix = scheme.pop("prefix")
        eprefix = scheme.pop("eprefix")
        sitedir = scheme.pop("sitedir")
        includedir = scheme.pop("includedir")

        self.assertEqual(prefix, "/usr/local")
        self.assertEqual(eprefix, "/usr/local")
        self.assertEqual(sitedir, "/usr/local/lib/python%s/dist-packages" % PY_VERSION_SHORT)
        self.assertEqual(includedir, "/usr/local/include/python%s/foo" % PY_VERSION_SHORT)

        scheme.pop("py_version_short")
        scheme.pop("pkgname")
        # Check that other values in scheme have not been modified
        for k, v in scheme.items():
            self.assertEqual(UNIX_REFERENCE[k], v)

    @mock.patch("sys.platform", "linux2")
    @mock.patch("bento.core.platforms.sysconfig.bento.utils.path.find_root", lambda ignored: r"/")
    @mock.patch("bento.commands.configure.virtualenv_prefix", lambda: "/home/guido/.env")
    def test_scheme_venv(self):
        bento_info = """\
Name: foo
"""

        scheme = self._compute_scheme(bento_info, self.options)
        prefix = scheme.pop("prefix")
        eprefix = scheme.pop("eprefix")

        self.assertEqual(prefix, "/home/guido/.env")
        self.assertEqual(eprefix, "/home/guido/.env")

        scheme.pop("py_version_short")
        scheme.pop("pkgname")
        # Check that other values in scheme have not been modified
        for k, v in scheme.items():
            self.assertEqual(UNIX_REFERENCE[k], v)

class TestWin32Scheme(unittest.TestCase):
    def setUp(self):
        super(TestWin32Scheme, self).setUp()
        options = mock.Mock()
        options.eprefix = None
        options.prefix = None

        self.options = options

    def _compute_scheme(self, bento_info, options):
        package_options = PackageOptions.from_string(bento_info)
        pkg = PackageDescription.from_string(bento_info)
        scheme = _compute_scheme(package_options)
        set_scheme_win32(scheme, options, pkg)

        return scheme

    @mock.patch("sys.platform", "win32")
    @mock.patch("bento.core.platforms.sysconfig.bento.utils.path.find_root", lambda ignored: "C:\\")
    @mock.patch("sys.prefix", r"C:\Python%s" % PY_VERSION_SHORT_NO_DOT)
    @mock.patch("sys.exec_prefix", r"C:\Python%s" % PY_VERSION_SHORT_NO_DOT)
    def test_scheme_default(self):
        bento_info = """\
Name: foo
"""
        self.options.prefix = self.eprefix = None

        scheme = self._compute_scheme(bento_info, self.options)
        prefix = scheme.pop("prefix")
        eprefix = scheme.pop("eprefix")
        py_version_short = scheme.pop("py_version_short")
        pkgname = scheme.pop("pkgname")

        self.assertEqual(prefix, sys.prefix)
        self.assertEqual(eprefix, sys.exec_prefix)
        self.assertEqual(pkgname, "foo")
        self.assertEqual(py_version_short, PY_VERSION_SHORT)

        # Check that other values in scheme have not been modified
        for k, v in scheme.items():
            self.assertEqual(WIN32_REFERENCE[k], v, "discrepency for path %s: %s vs %s" % (k, WIN32_REFERENCE[k], v))

    @mock.patch("sys.platform", "win32")
    @mock.patch("bento.core.platforms.sysconfig.bento.utils.path.find_root", lambda ignored: "C:\\")
    def test_scheme_prefix(self):
        bento_info = """\
Name: foo
"""
        self.options.prefix = r"C:\foo"
        self.eprefix = None

        scheme = self._compute_scheme(bento_info, self.options)
        prefix = scheme.pop("prefix")
        eprefix = scheme.pop("eprefix")
        scheme.pop("py_version_short")
        scheme.pop("pkgname")

        self.assertEqual(prefix, r"C:\foo")
        self.assertEqual(eprefix, r"C:\foo")

        # Check that other values in scheme have not been modified
        for k, v in scheme.items():
            self.assertEqual(WIN32_REFERENCE[k], v, "discrepency for path %s: %s vs %s" % (k, WIN32_REFERENCE[k], v))



import os
import sys
import builtins
import io
import py_compile
import marshal

if sys.version_info[:2] < (3, 2):
    def _bcompile(file, cfile=None, dfile=None, doraise=False):
        encoding = py_compile.read_encoding(file, "utf-8")
        f = open(file, 'U', encoding=encoding)
        try:
            timestamp = int(os.fstat(f.fileno()).st_mtime)
        except AttributeError:
            timestamp = int(os.stat(file).st_mtime)
        codestring = f.read()
        f.close()
        if codestring and codestring[-1] != '\n':
            codestring = codestring + '\n'
        try:
            codeobject = builtins.compile(codestring, dfile or file,'exec')
        except Exception as err:
            py_exc = py_compile.PyCompileError(err.__class__, err, dfile or file)
            if doraise:
                raise py_exc
            else:
                sys.stderr.write(py_exc.msg + '\n')
                return
        fc = io.BytesIO()
        try:
            fc.write(b'\0\0\0\0')
            py_compile.wr_long(fc, timestamp)
            marshal.dump(codeobject, fc)
            fc.flush()
            fc.seek(0, 0)
            fc.write(py_compile.MAGIC)
            return fc.getvalue()
        finally:
            fc.close()
else:
    import tokenize
    import imp
    import errno
    def _bcompile(file, cfile=None, dfile=None, doraise=False, optimize=-1):
        with tokenize.open(file) as f:
            try:
                timestamp = int(os.fstat(f.fileno()).st_mtime)
            except AttributeError:
                timestamp = int(os.stat(file).st_mtime)
            codestring = f.read()
        try:
            codeobject = builtins.compile(codestring, dfile or file, 'exec',
                                          optimize=optimize)
        except Exception as err:
            py_exc = py_compile.PyCompileError(err.__class__, err, dfile or file)
            if doraise:
                raise py_exc
            else:
                sys.stderr.write(py_exc.msg + '\n')
                return
        if cfile is None:
            if optimize >= 0:
                cfile = imp.cache_from_source(file, debug_override=not optimize)
            else:
                cfile = imp.cache_from_source(file)
        try:
            os.makedirs(os.path.dirname(cfile))
        except OSError as error:
            if error.errno != errno.EEXIST:
                raise
        fc = io.BytesIO()
        try:
            fc.write(b'\0\0\0\0')
            py_compile.wr_long(fc, timestamp)
            marshal.dump(codeobject, fc)
            fc.flush()
            fc.seek(0, 0)
            fc.write(py_compile.MAGIC)
            return fc.getvalue()
        finally:
            fc.close()

def bcompile(file):
    return _bcompile(file, doraise=True)



import sys
import os
import copy
import distutils
import distutils.sysconfig
import re
import warnings
import errno

from subprocess \
    import \
        Popen, PIPE, STDOUT

from yaku.task_manager \
    import \
        topo_sort, build_dag, \
        CompiledTaskGen, set_extension_hook
from yaku.sysconfig \
    import \
        get_configuration, detect_distutils_cc
from yaku.compiled_fun \
    import \
        compile_fun
from yaku.task \
    import \
        task_factory
from yaku.utils \
    import \
        ensure_dir, get_exception
from yaku.environment \
    import \
        Environment
from yaku.conftests \
    import \
        check_compiler, check_header
from yaku.tools.ctasks \
    import \
        apply_define
from yaku.scheduler \
    import \
        run_tasks
from yaku.conf \
    import \
        with_conf_blddir, create_file, write_log
from yaku.errors \
    import \
        TaskRunFailure
from yaku._config \
    import \
        _OUTPUT
import yaku.tools

pylink, pylink_vars = compile_fun("pylink", "${PYEXT_SHLINK} ${PYEXT_LINK_TGT_F}${TGT[0].abspath()} ${PYEXT_LINK_SRC_F}${SRC} ${PYEXT_APP_LIBDIR} ${PYEXT_APP_LIBS} ${PYEXT_APP_FRAMEWORKS} ${PYEXT_SHLINKFLAGS}", False)

pycc, pycc_vars = compile_fun("pycc", "${PYEXT_CC} ${PYEXT_CFLAGS} ${PYEXT_INCPATH} ${PYEXT_CC_TGT_F}${TGT[0].abspath()} ${PYEXT_CC_SRC_F}${SRC}", False)

pycxx, pycxx_vars = compile_fun("pycxx", "${PYEXT_CXX} ${PYEXT_CXXFLAGS} ${PYEXT_INCPATH} ${PYEXT_CXX_TGT_F}${TGT[0].abspath()} ${PYEXT_CXX_SRC_F}${SRC}", False)

pycxxlink, pycxxlink_vars = compile_fun("pycxxlink", "${PYEXT_CXXSHLINK} ${PYEXT_LINK_TGT_F}${TGT[0].abspath()} ${PYEXT_LINK_SRC_F}${SRC} ${PYEXT_APP_LIBDIR} ${PYEXT_APP_LIBS} ${PYEXT_APP_FRAMEWORKS} ${PYEXT_SHLINKFLAGS}", False)

# pyext env <-> sysconfig env conversion

_SYS_TO_PYENV = {
        "PYEXT_SHCC": "CC",
        "PYEXT_CCSHARED": "CCSHARED",
        "PYEXT_SHLINK": "LDSHARED",
        "PYEXT_SUFFIX": "SO",
        "PYEXT_CFLAGS": "CFLAGS",
        "PYEXT_OPT": "OPT",
        "PYEXT_LIBDIR": "LIBDIR",
}

_PYENV_REQUIRED = [
        "LIBDIR_FMT",
        "LIBS",
        "LIB_FMT",
        "CPPPATH_FMT",
        "CC_TGT_F",
        "CC_SRC_F",
        "LINK_TGT_F",
        "LINK_SRC_F",
]

_SYS_TO_CCENV = {
        "CC": "CC",
        "SHCC": "CCSHARED",
        "SHLINK": "LDSHARED",
        "SO": "SO",
        "CFLAGS": "CFLAGS",
        "OPT": "OPT",
        "LIBDIR": "LIBDIR",
        "LIBDIR_FMT": "LIBDIR_FMT",
        "LIBS": "LIBS",
        "LIB_FMT": "LIB_FMT",
        "CPPPATH_FMT": "CPPPATH_FMT",
        "CC_TGT_F": "CC_TGT_F",
        "CC_SRC_F": "CC_SRC_F",
        "CXX": "CXX",
        "CXXSHLINK": "CXXSHLINK",
}

def setup_pyext_env(ctx, cc_type="default", use_distutils=True):
    pyenv = Environment()
    if use_distutils:
        if cc_type == "default":
            dist_env = get_configuration()
        else:
            dist_env = get_configuration(cc_type)
        for name, value in dist_env.items():
            pyenv["PYEXT_%s" % name] = value
        pyenv["PYEXT_FMT"] = "%%s%s" % dist_env["SO"]
        pyenv["PYEXT_SHLINKFLAGS"] = dist_env["LDFLAGS"]
    else:
        old_env = ctx.env
        ctx.env = Environment()
        cc_env = None
        sys.path.insert(0, os.path.dirname(yaku.tools.__file__))
        try:
            try:
                mod = __import__(cc_type)
                mod.setup(ctx)
            except ImportError:
                raise RuntimeError("No tool %s is available (import failed)" \
                                % cc_type)
            cc_env = ctx.env
        finally:
            sys.path.pop(0)
            ctx.env = old_env
        pyenv["PYEXT_CC"] = cc_env["CC"]
        pyenv["PYEXT_CFLAGS"] = cc_env["CFLAGS"]
        pyenv["PYEXT_LIBDIR"] = cc_env["LIBDIR"]
        pyenv["PYEXT_LIBS"] = cc_env["LIBS"]
        pyenv["PYEXT_FMT"] = "%s.so"
        pyenv["PYEXT_SHLINK"] = cc_env["MODLINK"]
        pyenv["PYEXT_SHLINKFLAGS"] = cc_env["MODLINKFLAGS"]
        pyenv["PYEXT_CPPPATH"] = cc_env["CPPPATH"]
        pyenv.append("PYEXT_CPPPATH", distutils.sysconfig.get_python_inc(), create=True)
        if sys.platform == "win32":
            pyenv.append("PYEXT_LIBDIR", os.path.join(sys.exec_prefix, "libs"))

    return pyenv

def pycc_hook(self, node):
    tasks = pycc_task(self, node)
    self.object_tasks.extend(tasks)
    return tasks

def pycc_task(self, node):
    base = self.env["CC_OBJECT_FMT"] % node.name
    target = node.parent.declare(base)
    ensure_dir(target.abspath())

    task = task_factory("pycc")(inputs=[node], outputs=[target])
    task.gen = self
    task.env_vars = pycc_vars
    task.env = self.env
    task.func = pycc
    return [task]

def pycxx_hook(self, node):
    tasks = pycxx_task(self, node)
    self.object_tasks.extend(tasks)
    self.has_cxx = True
    return tasks

def pycxx_task(self, node):
    base = self.env["CXX_OBJECT_FMT"] % node.name
    target = node.parent.declare(base)
    ensure_dir(target.abspath())

    task = task_factory("pycxx")(inputs=[node], outputs=[target])
    task.gen = self
    task.env_vars = pycxx_vars
    task.env = self.env
    task.func = pycxx
    return [task]

def pylink_task(self, name):
    objects = [tsk.outputs[0] for tsk in self.object_tasks]
    if len(objects) < 1:
        warnings.warn("task %s has no inputs !" % name)
    def declare_target():
        folder, base = os.path.split(name)
        tmp = folder + os.path.sep + self.env["PYEXT_FMT"] % base
        return self.bld.path.declare(tmp)
    target = declare_target()
    ensure_dir(target.abspath())

    task = task_factory("pylink")(inputs=objects, outputs=[target])
    task.gen = self
    task.func = pylink
    task.env_vars = pylink_vars
    self.link_task = task

    return [task]

# XXX: fix merge env location+api
class PythonBuilder(yaku.tools.Builder):
    def clone(self):
        return PythonBuilder(self.ctx)

    def __init__(self, ctx):
        yaku.tools.Builder.__init__(self, ctx)

    def _compile(self, task_gen, name):
        apply_define(task_gen)
        apply_cpppath(task_gen)

        tasks = task_gen.process()
        for t in tasks:
            t.env = task_gen.env
        return tasks

    def try_compile(self, name, body, headers=None):
        old_hook = set_extension_hook(".c", pycc_task)
        try:
            return with_conf_blddir(self.ctx, name, body,
                                    lambda : yaku.tools.try_task_maker(self.ctx, self._compile, name, body, headers))
        finally:
            set_extension_hook(".c", old_hook)

    def try_extension(self, name, body, headers=None):
        old_hook = set_extension_hook(".c", pycc_task)
        try:
            return with_conf_blddir(self.ctx, name, body,
                                    lambda : yaku.tools.try_task_maker(self.ctx, self._extension, name, body, headers))
        finally:
            set_extension_hook(".c", old_hook)

    def _extension(self, task_gen, name):
        bld = self.ctx
        base = name.replace(".", os.sep)

        tasks = []

        old_hook = set_extension_hook(".c", pycc_hook)
        old_hook_cxx = set_extension_hook(".cxx", pycxx_hook)

        apply_define(task_gen)
        apply_cpppath(task_gen)
        apply_libpath(task_gen)
        apply_libs(task_gen)
        apply_frameworks(task_gen)

        tasks = task_gen.process()

        ltask = pylink_task(task_gen, base)
        task_gen.link_task = ltask
        if task_gen.has_cxx:
            task_gen.link_task[-1].func = pycxxlink
            task_gen.link_task[-1].env_vars = pycxxlink_vars

        tasks.extend(ltask)
        for t in tasks:
            t.env = task_gen.env

        set_extension_hook(".c", old_hook)
        set_extension_hook(".cxx", old_hook_cxx)
        return tasks

    def extension(self, name, sources, env=None):
        sources = self.to_nodes(sources)
        task_gen = CompiledTaskGen("pyext", self.ctx, sources, name)
        task_gen.bld = self.ctx
        task_gen.env = yaku.tools._merge_env(self.env, env)
        tasks = self._extension(task_gen, name)
        self.ctx.tasks.extend(tasks)

        outputs = []
        for t in task_gen.link_task:
            outputs.extend(t.outputs)
        task_gen.outputs = outputs
        return tasks

    def try_extension(self, name, body, headers=None):
        return with_conf_blddir(self.ctx, name, body,
                                lambda : yaku.tools.try_task_maker(self.ctx, self._extension, name, body, headers))

    def configure(self, candidates=None, use_distutils=True):
        ctx = self.ctx
        # How we do it
        # 1: for distutils-based configuration
        #   - get compile/flags flags from sysconfig
        #   - detect yaku tool name from CC used by distutils:
        #       - get the compiler executable used by distutils ($CC
        #       variable)
        #       - try to determine yaku tool name from $CC
        #   - apply necessary variables from yaku tool to $PYEXT_
        #   "namespace"
        if candidates is None:
            compiler_type = "default"
        else:
            compiler_type = candidates[0]

        if use_distutils:
            dist_env = setup_pyext_env(ctx, compiler_type)
            ctx.env.update(dist_env)

            cc_exec = get_distutils_cc_exec(ctx, compiler_type)
            yaku_cc_type = detect_cc_type(ctx, cc_exec)
            if yaku_cc_type is None:
                raise ValueError("No adequate C compiler found (distutils mode)")

            _setup_compiler(ctx, yaku_cc_type)

            cxx_exec = get_distutils_cxx_exec(ctx, compiler_type)
            yaku_cxx_type = detect_cxx_type(ctx, cxx_exec)
            if yaku_cxx_type is None:
                raise ValueError("No adequate CXX compiler found (distutils mode)")

            _setup_cxxcompiler(ctx, yaku_cxx_type)
        else:
            dist_env = setup_pyext_env(ctx, compiler_type, False)
            ctx.env.update(dist_env)
            _setup_compiler(ctx, compiler_type)

        pycode = r"""\
#include <Python.h>
#include <stdio.h>

static PyObject*
hello(PyObject *self, PyObject *args)
{
    printf("Hello from C\n");
    Py_INCREF(Py_None);
    return Py_None;
}

static PyMethodDef HelloMethods[] = {
    {"hello",  hello, METH_VARARGS, "Print a hello world."},
    {NULL, NULL, 0, NULL}        /* Sentinel */
};

PyMODINIT_FUNC
init_bar(void)
{
    (void) Py_InitModule("_bar", HelloMethods);
}
"""
        ctx.start_message("Checking whether %s can build python object code" % compiler_type)
        try:
            self.try_compile("foo", pycode)
            ctx.end_message("yes")
        except TaskRunFailure:
            e = get_exception()
            ctx.end_message("no")
            ctx.fail_configuration(str(e))

        ctx.start_message("Checking whether %s can build python extension" % compiler_type)
        try:
            self.try_extension("foo", pycode)
            ctx.end_message("yes")
        except TaskRunFailure:
            e = get_exception()
            ctx.end_message("no")
            ctx.fail_configuration(str(e))
        self.configured = True

def get_builder(ctx):
    return PythonBuilder(ctx)

CC_SIGNATURE = {
        "clang": re.compile("clang version"),
        "gcc": re.compile("gcc version"),
        "msvc": re.compile("Microsoft \(R\) (32-bit |)C/C\+\+ Optimizing Compiler")
}

def detect_cc_type(ctx, cc_cmd):
    return _detect_cc_type(ctx, cc_cmd)

def detect_cxx_type(ctx, cxx_cmd):
    cxx_type = _detect_cc_type(ctx, cxx_cmd)
    if cxx_type == "gcc":
        return "gxx"
    else:
        return cxx_type

def _detect_cc_type(ctx, cc_cmd):
    cc_type = None

    def detect_type(vflag):
        cmd = cc_cmd + [vflag]
        env = os.environ
        env["LC_ALL"] = env["LANGUAGE"] = "C"
        p = Popen(cmd, stdout=PIPE, stderr=STDOUT, env=env)
        out = p.communicate()[0].decode()
        for k, v in CC_SIGNATURE.items():
            m = v.search(out)
            if m:
                return k
        return None

    _OUTPUT.write("Detecting CC type... ")
    if sys.platform == "win32":
        for v in ["", "-v"]:
            cc_type = detect_type(v)
    else:
        try:
            for v in ["-v", "-V", "-###"]:
                cc_type = detect_type(v)
                if cc_type:
                    break
            if cc_type is None:
                cc_type = "cc"
        except OSError:
            e = extract_exception()
            if e.errno == errno.ENOENT:
                raise ValueError("compiler %r not found" % " ".join(cc_cmd))
            else:
                raise ValueError("Unexpected error %r when testing compiler %r" % (e, cc_cmd))
    _OUTPUT.write("%s\n" % cc_type)
    return cc_type

def get_distutils_cc_exec(ctx, compiler_type="default"):
    from distutils import ccompiler
    from distutils.sysconfig import customize_compiler

    _OUTPUT.write("Detecting distutils CC exec ... ")
    if compiler_type == "default":
        compiler_type = \
                distutils.ccompiler.get_default_compiler()

    compiler = ccompiler.new_compiler(compiler=compiler_type)
    customize_compiler(compiler)
    if compiler_type == "msvc":
        compiler.initialize()
        cc = [compiler.cc]
    else:
        cc = compiler.compiler_so[:1]
    _OUTPUT.write("%s\n" % " ".join(cc))
    return cc

def get_distutils_cxx_exec(ctx, compiler_type="default"):
    from distutils import ccompiler
    from distutils.sysconfig import customize_compiler

    _OUTPUT.write("Detecting distutils CXX exec ... ")
    if compiler_type == "default":
        compiler_type = \
                distutils.ccompiler.get_default_compiler()

    compiler = ccompiler.new_compiler(compiler=compiler_type)
    if compiler_type == "msvc":
        compiler.initialize()
        cc = [compiler.cc]
    else:
        customize_compiler(compiler)
        cc = compiler.compiler_cxx
    _OUTPUT.write("%s\n" % " ".join(cc))
    return cc

def _setup_compiler(ctx, cc_type):
    old_env = ctx.env
    ctx.env = Environment()
    cc_env = None
    sys.path.insert(0, os.path.dirname(yaku.tools.__file__))
    try:
        try:
            mod = __import__(cc_type)
            mod.setup(ctx)
        except ImportError:
            raise RuntimeError("No tool %s is available (import failed)" \
                            % cc_type)

        # XXX: this is ugly - find a way to have tool-specific env...
        cc_env = ctx.env
    finally:
        sys.path.pop(0)
        ctx.env = old_env

    copied_values = ["CC", "CPPPATH_FMT", "LIBDIR_FMT", "LIB_FMT",
            "CC_OBJECT_FMT", "CC_TGT_F", "CC_SRC_F", "LINK_TGT_F",
            "LINK_SRC_F", "SHLINK"]
    for k in copied_values:
        ctx.env["PYEXT_%s" % k] = cc_env[k]
    ctx.env.prextend("PYEXT_CPPPATH", cc_env["CPPPATH"])
    ctx.env.prextend("PYEXT_LIBDIR", cc_env["LIBDIR"])

def _setup_cxxcompiler(ctx, cxx_type):
    old_env = ctx.env
    ctx.env = Environment()
    sys.path.insert(0, os.path.dirname(yaku.tools.__file__))
    try:
        mod = __import__(cxx_type)
        mod.setup(ctx)
        cxx_env = ctx.env
    finally:
        sys.path.pop(0)
        ctx.env = old_env

    for k in ["CXX", "CXXFLAGS", "CXX_TGT_F", "CXX_SRC_F",
              "CXXSHLINK"]:
        ctx.env["PYEXT_%s" % k] = cxx_env[k]

# FIXME: find a way to reuse this kind of code between tools
def apply_frameworks(task_gen):
    # XXX: do this correctly (platform specific tool config)
    if sys.platform == "darwin":
        frameworks = task_gen.env.get("PYEXT_FRAMEWORKS", [])
        task_gen.env["PYEXT_APP_FRAMEWORKS"] = []
        for framework in frameworks:
            task_gen.env["PYEXT_APP_FRAMEWORKS"].extend(["-framework", framework])
    else:
        task_gen.env["PYEXT_APP_FRAMEWORKS"] = []

def apply_libs(task_gen):
    libs = task_gen.env["PYEXT_LIBS"]
    task_gen.env["PYEXT_APP_LIBS"] = [
            task_gen.env["PYEXT_LIB_FMT"] % lib for lib in libs]

def apply_libpath(task_gen):
    libdir = task_gen.env["PYEXT_LIBDIR"]
    #implicit_paths = set([
    #    os.path.join(task_gen.env["BLDDIR"], os.path.dirname(s))
    #    for s in task_gen.sources])
    implicit_paths = []
    libdir = list(implicit_paths) + libdir
    task_gen.env["PYEXT_APP_LIBDIR"] = [
            task_gen.env["PYEXT_LIBDIR_FMT"] % d for d in libdir]

def apply_cpppath(task_gen):
    cpppaths = task_gen.env["PYEXT_CPPPATH"]
    implicit_paths = set([s.parent.srcpath() \
                          for s in task_gen.sources])
    srcnode = task_gen.sources[0].ctx.srcnode

    relcpppaths = []
    for p in cpppaths:
        if not os.path.isabs(p):
            node = srcnode.find_node(p)
            assert node is not None, "could not find %s" % p
            relcpppaths.append(node.bldpath())
        else:
            relcpppaths.append(p)
    cpppaths = list(implicit_paths) + relcpppaths
    task_gen.env["PYEXT_INCPATH"] = [
            task_gen.env["PYEXT_CPPPATH_FMT"] % p
            for p in cpppaths]



import os
import shutil
import tempfile

import os.path as op

from bento.compat.api.moves \
    import \
        unittest
from bento.core \
    import \
        PackageDescription, PackageOptions
from bento.errors \
    import \
        InvalidPackage
from bento.core.node \
    import \
        create_root_with_source_tree
from bento.core.node_package \
    import \
        NodeRepresentation
from bento.commands.hooks \
    import \
        create_hook_module, find_pre_hooks
from bento.commands.wrapper_utils\
    import \
        run_command_in_context
from bento.backends.yaku_backend \
    import \
        ConfigureYakuContext
from bento.installed_package_description \
    import \
        InstalledSection

from bento.core.testing \
    import \
        create_fake_package_from_bento_infos
from bento.commands.tests.utils \
    import \
        prepare_configure, prepare_build, comparable_installed_sections

def comparable_representation(top_node, node_pkg):
    """Return a dictionary representing the node_pkg to be used for
    comparison."""
    d = {"packages": {}, "extensions": {}}
    for k, v in node_pkg.iter_category("extensions"):
        d["extensions"][k] = v.extension_from(top_node)
    for k, v in node_pkg.iter_category("packages"):
        d["packages"][k] = (v.full_name, v.nodes, v.top_node, v.top_or_lib_node)
    return d

class TestRecurseBase(unittest.TestCase):
    def setUp(self):
        self.old_dir = os.getcwd()

        self.d = tempfile.mkdtemp()
        self.root = create_root_with_source_tree(self.d, os.path.join(self.d, "build"))
        self.run_node = self.root.find_node(self.d)
        self.top_node = self.run_node

        os.chdir(self.d)

    def tearDown(self):
        os.chdir(self.old_dir)
        shutil.rmtree(self.d)

    def _create_package_and_reference(self, bento_info, r_bento_info):
        pkg = PackageDescription.from_string(bento_info)
        node_pkg = NodeRepresentation(self.run_node, self.top_node)
        node_pkg.update_package(pkg)

        r_pkg = PackageDescription.from_string(r_bento_info)
        r_node_pkg = NodeRepresentation(self.run_node, self.top_node)
        r_node_pkg.update_package(r_pkg)

        return node_pkg, r_node_pkg

    def test_py_packages(self):
        run_node = self.run_node

        bento_info = """\
Name: foo

Recurse: bar

Library:
    Packages: bar
"""
        sub_bento_info = """\
Library:
    Packages: foo
"""

        r_bento_info = """\
Name: foo

Library:
    Packages: bar, bar.foo
"""

        bentos = {"bento.info": bento_info,
                  op.join("bar", "bento.info"): sub_bento_info}
        create_fake_package_from_bento_infos(run_node, bentos)

        node_pkg, r_node_pkg = self._create_package_and_reference(bento_info, r_bento_info)
        self.assertEqual(comparable_representation(self.top_node, node_pkg),
                         comparable_representation(self.top_node, r_node_pkg))


    def test_extension(self):
        run_node = self.run_node

        bento_info = """\
Name: foo

Recurse: bar

Library:
    Extension: foo
        Sources: src/foo.c
"""
        sub_bento_info = """\
Library:
    Extension: foo
        Sources: src/foo.c
"""

        r_bento_info = """\
Name: foo

Library:
    Extension: foo
        Sources: src/foo.c
    Extension: bar.foo
        Sources: bar/src/foo.c
"""

        bentos = {"bento.info": bento_info,
                  op.join("bar", "bento.info"): sub_bento_info}
        create_fake_package_from_bento_infos(run_node, bentos)

        node_pkg, r_node_pkg = self._create_package_and_reference(bento_info, r_bento_info)
        self.assertEqual(comparable_representation(self.top_node, node_pkg),
                         comparable_representation(self.top_node, r_node_pkg))

    def test_basics(self):
        run_node = self.run_node

        bento_info = """\
Name: foo

Recurse:
    bar
"""
        bento_info2 = """\
Recurse:
    foo

Library:
    Extension: _foo
        Sources: foo.c
    CompiledLibrary: _bar
        Sources: foo.c
"""

        bento_info3 = """\
Library:
    Packages: sub2
"""
        bentos = {"bento.info": bento_info, os.path.join("bar", "bento.info"): bento_info2,
                  os.path.join("bar", "foo", "bento.info"): bento_info3}
        create_fake_package_from_bento_infos(run_node, bentos)

        r_bento_info = """\
Name: foo

Library:
    Packages:
        bar.foo.sub2
    Extension: bar._foo
        Sources: bar/foo.c
    CompiledLibrary: bar._bar
        Sources: bar/foo.c
"""

        node_pkg, r_node_pkg = self._create_package_and_reference(bento_info, r_bento_info)

        self.assertEqual(comparable_representation(self.top_node, node_pkg),
                         comparable_representation(self.top_node, r_node_pkg))

    def test_py_module_invalid(self):
        """Ensure we get a package error when defining py modules in recursed
        bento.info."""
        bento_info = """\
Name: foo

Recurse: bar
"""
        sub_bento_info = """\
Library:
    Modules: foo
"""
        bentos = {"bento.info": bento_info,
                  os.path.join("bar", "bento.info"): sub_bento_info}
        self.assertRaises(InvalidPackage,
                          lambda: create_fake_package_from_bento_infos(self.run_node, bentos))

    def test_hook(self):
        root = self.root
        top_node = self.top_node

        bento_info = """\
Name: foo

HookFile:
    bar/bscript

Recurse:
    bar
"""
        bento_info2 = """\
Library:
    Packages: fubar
"""

        bscript = """\
from bento.commands import hooks
@hooks.pre_configure
def configure(ctx):
    packages = ctx.local_pkg.packages
    ctx.local_node.make_node("test").write(str(packages))
"""
        bentos = {"bento.info": bento_info, os.path.join("bar", "bento.info"): bento_info2}
        bscripts = {os.path.join("bar", "bscript"): bscript}
        create_fake_package_from_bento_infos(top_node, bentos, bscripts)

        conf, configure = prepare_configure(self.run_node, bento_info, ConfigureYakuContext)
        try:
            hook = top_node.search("bar/bscript")
            m = create_hook_module(hook.abspath())
            for hook in find_pre_hooks([m], "configure"):
                conf.pre_recurse(root.find_dir(hook.local_dir))
                try:
                    hook(conf)
                finally:
                    conf.post_recurse()

            test = top_node.search("bar/test")
            if test:
                self.failUnlessEqual(test.read(), "['fubar']")
            else:
                self.fail("test dummy not found")
        finally:
            configure.finish(conf)
            conf.finish()

class TestInstalledSections(unittest.TestCase):
    """Test registered installed sections are the expected ones when using
    recursive support."""
    def setUp(self):
        self.old_dir = os.getcwd()

        self.d = tempfile.mkdtemp()
        self.root = create_root_with_source_tree(self.d, os.path.join(self.d, "build"))
        self.run_node = self.root.find_node(self.d)
        self.top_node = self.run_node

        os.chdir(self.d)

    def tearDown(self):
        os.chdir(self.old_dir)
        shutil.rmtree(self.d)

    def _test_installed_sections(self, bento_infos, r_sections):
        create_fake_package_from_bento_infos(self.top_node, bento_infos)

        conf, configure = prepare_configure(self.run_node, bento_infos["bento.info"])
        run_command_in_context(conf, configure)

        bld, build = prepare_build(self.top_node, bento_infos["bento.info"])
        run_command_in_context(bld, build)

        sections = bld.section_writer.sections

        self.assertEqual(comparable_installed_sections(sections),
                         comparable_installed_sections(r_sections))

    def test_packages(self):
        bento_info = """\
Name: foo

Recurse:
    foo

Library:
    Packages: foo
"""
        sub_bento_info = """\
Library:
    Packages: bar
"""

        r_bar_section = InstalledSection.from_source_target_directories("pythonfiles",
                                     "foo",
                                     "$_srcrootdir/..",
                                     "$sitedir",
                                     ["foo/__init__.py"])
        r_foo_bar_section = InstalledSection.from_source_target_directories("pythonfiles",
                                     "foo.bar",
                                     "$_srcrootdir/..",
                                     "$sitedir",
                                     ["foo/bar/__init__.py"])
        r_sections = {"pythonfiles":
                         {"foo": r_bar_section,
                          "foo.bar": r_foo_bar_section}}
        self._test_installed_sections({"bento.info": bento_info, op.join("foo", "bento.info"): sub_bento_info},
                                      r_sections)

    def test_extension(self):
        bento_info = """\
Name: foo

Recurse: bar
"""

        sub_bento_info = """\
Library:
    Extension: foo
        Sources: src/foo.c
"""

        r_section = InstalledSection.from_source_target_directories("extensions",
                                     "bar.foo",
                                     "$_srcrootdir/bar",
                                     "$sitedir/bar",
                                     ["foo.so"])
        r_sections = {"extensions":
                        {"bar.foo":
                            r_section}}
        self._test_installed_sections({"bento.info": bento_info, op.join("bar", "bento.info"): sub_bento_info},
                                      r_sections)



import sys
import os
import os.path as op
import warnings

from distutils.cmd \
    import \
        Command
from distutils.command.install \
    import \
        INSTALL_SCHEMES

from bento._config \
    import \
        IPKG_PATH
from bento.installed_package_description \
    import \
        InstalledPkgDescription, iter_files
from bento.utils.utils import subst_vars
import bento.utils.io2

class install(Command):
    cmd_name = "install"
    description = "Install wrapper to install bento package"

    user_options = [
        ('prefix=', None, "installation prefix"),
        ('exec-prefix=', None, "prefix for platform-specific files"),

        ('record=', None, "Record file containing every path created by install command"),
        ('root=', None, "(Unix-only) Alternative root (equivalent to destdir option in bento)"),

        ('dry-run', 'n', "(Unix-only) Alternative root (equivalent to destdir option in bento)"),
        ('single-version-externally-managed', None, "Do nothing. For compatibility with pip only."),
        ('install-headers=', None, "Do nothing. For compatibility with pip only."),
        ('force', None, "Do nothing. For compatibility with distutils."),
    ]

    def initialize_options(self):
        self.prefix = None
        self.exec_prefix = None
        self.record = None
        self.root = None
        self.dry_run = None
        self.install_headers = None
        self.single_version_externally_managed = None

        self.scheme = {}

    def finalize_options(self):
        if self.install_headers is not None:
            warnings.warn("--install-headers option is ignored.")

        if self.prefix is None:
            prefix_customized = False
        else:
            prefix_customized = True

        if self.prefix is None:
            if self.exec_prefix is not None:
                raise DistutilsOptionError("must not supply exec-prefix without prefix")

            self.prefix = os.path.normpath(sys.prefix)
            self.exec_prefix = os.path.normpath(sys.exec_prefix)
        else:
            if self.exec_prefix is None:
                self.exec_prefix = self.prefix

        if os.name == "posix":
            self._finalize_unix(prefix_customized)
        else:
            self._finalize_other(prefix_customized)

    def _finalize_other(self, is_prefix_customized):
        scheme = self.scheme

        if self.root:
            raise ValueError("Option root is meaningless on non-posix platforms !")

        if is_prefix_customized:
            prefix = self.prefix
            exec_prefix = self.exec_prefix
        else:
            prefix = sys.prefix
            exec_prefix = sys.exec_prefix
        scheme['prefix'] = prefix
        scheme['exec_prefix'] = exec_prefix

    def _finalize_unix(self, is_prefix_customized):
        scheme = self.scheme

        if self.root:
            scheme["destdir"] = self.root

        # TODO: user and home schemes

        py_version_short = ".".join(map(str, sys.version_info[:2]))
        dist_name = self.distribution.pkg.name

        if is_prefix_customized:
            if op.normpath(self.prefix) != '/usr/local':
                # unix prefix
                scheme['prefix'] = self.prefix
                scheme['exec_prefix'] = self.exec_prefix
            else:
                scheme['prefix'] = self.prefix
                scheme['exec_prefix'] = self.exec_prefix
                # use deb_system on debian-like systems
                if 'deb_system' in INSTALL_SCHEMES:
                    v = {'py_version_short': py_version_short, 'dist_name': dist_name, 'base': self.prefix}
                    scheme['includedir'] = subst_vars(INSTALL_SCHEMES['deb_system']['headers'], v)
                    scheme['sitedir'] = subst_vars(INSTALL_SCHEMES['deb_system']['purelib'], v)
        else:
            # If no prefix is specified, we'd like to avoid installing anything
            # in /usr by default (i.e. like autotools, everything in
            # /usr/local).  If prefix is not /usr, then we can't really guess
            # the best default location.
            if hasattr(sys, 'real_prefix'): # run under virtualenv
                prefix = sys.prefix
                exec_prefix = sys.exec_prefix
            elif sys.prefix == '/usr':
                prefix = exec_prefix = '/usr/local'
            else:
                prefix = sys.prefix
                exec_prefix = sys.exec_prefix
            scheme['prefix'] = prefix
            scheme['exec_prefix'] = exec_prefix
            # use unix_local on debian-like systems
            if 'unix_local' in INSTALL_SCHEMES:
                v = {'py_version_short': py_version_short, 'dist_name': dist_name, 'base': prefix}
                scheme['includedir'] = subst_vars(INSTALL_SCHEMES['unix_local']['headers'], v)
                scheme['sitedir'] = subst_vars(INSTALL_SCHEMES['unix_local']['purelib'], v)

    def run(self):
        self.run_command("build")
        args = []

        if self.dry_run == 1:
            args.append("--dry-run")
        self.distribution.run_command_in_context(self.cmd_name, args)
        if self.record:
            self.write_record()

    def write_record(self):
        dist = self.distribution

        options_context = dist.global_context.retrieve_options_context(self.cmd_name)
        cmd_context_klass = dist.global_context.retrieve_command_context(self.cmd_name)
        context = cmd_context_klass(dist.global_context, [], options_context, dist.pkg, dist.run_node)

        n = context.build_node.make_node(IPKG_PATH)
        ipkg = InstalledPkgDescription.from_file(n.abspath())
        scheme = context.retrieve_configured_scheme()
        ipkg.update_paths(scheme)
        file_sections = ipkg.resolve_paths_with_destdir(src_root_node=context.build_node)

        def writer(fid):
            for kind, source, target in iter_files(file_sections):
                fid.write("%s\n" % target.abspath())
        bento.utils.io2.safe_write(self.record, writer, "w")



import os
import tempfile
import shutil

from bento.compat.api.moves \
    import \
        unittest

from bento.core.package \
    import \
        PackageDescription
from bento.core.node_package \
    import \
        NodeRepresentation
from bento.core.node \
    import \
        create_root_with_source_tree
from bento.core.testing \
    import \
        create_fake_package_from_bento_info

class TestNodeRepresentation(unittest.TestCase):
    def setUp(self):
        self.d = tempfile.mkdtemp()
        self.root = create_root_with_source_tree(self.d, os.path.join(self.d, "build"))
        self.top_node = self.root.find_node(self.d)

        self.old_dir = os.getcwd()
        os.chdir(self.d)

    def tearDown(self):
        os.chdir(self.old_dir)
        shutil.rmtree(self.d)

    def test_simple(self):
        bento_info = """\
Name: foo

Library:
    Extension: _foo
        Sources: src/foo.c, src/bar.c
"""
        create_fake_package_from_bento_info(self.top_node, bento_info)

        bento_info = """\
Name: foo

Library:
    Extension: _foo
        Sources: src/foo.c, src/bar.c
"""
        pkg = PackageDescription.from_string(bento_info)
        node_pkg = NodeRepresentation(self.top_node, self.top_node)
        node_pkg.update_package(pkg)

        extensions = dict(node_pkg.iter_category("extensions"))
        self.assertEqual(len(extensions), 1)
        self.assertEqual(len(extensions["_foo"].nodes), 2)

    def test_sources_glob(self):
        bento_info = """\
Name: foo

Library:
    Extension: _foo
        Sources: src/foo.c, src/bar.c
"""
        create_fake_package_from_bento_info(self.top_node, bento_info)

        bento_info = """\
Name: foo

Library:
    Extension: _foo
        Sources: src/*.c
"""
        pkg = PackageDescription.from_string(bento_info)
        node_pkg = NodeRepresentation(self.top_node, self.top_node)
        node_pkg.update_package(pkg)

        extensions = dict(node_pkg.iter_category("extensions"))
        self.assertEqual(len(extensions), 1)
        self.assertEqual(len(extensions["_foo"].nodes), 2)



"""Option handling module.

This module is concerned with options/argument parsing from both
bento and bento subcommands, as well as help message
formatting."""
import optparse

from optparse \
    import \
        Option

class IndentedHelpFormatter(optparse.IndentedHelpFormatter):
    def format_usage(self, usage):
        return optparse._("%s\n" % usage)

# Goal of separate options context:
#   - separate options handling from commands themselves (simplify commands)
#   - easier to add options from hook files
#   - should help hiding command implementation detail from hooks and high
#   level tools such as bentomaker (close coupling at the moment)
class OptionsContext(object):
    @classmethod
    def from_command(cls, cmd):
        usage = cmd.long_descr
        ret = cls(usage=usage)
        for o in cmd.common_options:
            ret.add_option(o)
        return ret

    def __init__(self, usage=None):
        kw = {"add_help_option": False, "formatter": IndentedHelpFormatter()}
        if usage is not None:
            kw["usage"] = usage
        self.parser = optparse.OptionParser(**kw)
        self._groups = {}
        self._is_setup = False

    def setup(self, package_options):
        self.add_group("build_customization", "Build customization")
        opt = optparse.Option("--use-distutils", help="Build extensions with distutils",
                              action="store_true")
        self.add_option(opt, "build_customization")

        self._is_setup = True

    def add_option(self, option, group=None):
        if group is None:
            self.parser.add_option(option)
        else:
            if group in self._groups:
                self._groups[group].add_option(option)
            else:
                raise ValueError("Unknown option group %r" % group)

    def has_group(self, group):
        return group in self._groups

    def add_group(self, name, title):
        grp = optparse.OptionGroup(self.parser, title)
        self._groups[name] = grp
        self.parser.add_option_group(grp)



from bento.private.version \
    import \
        NormalizedVersion, is_valid_version, suggest_normalized_version
from bento.errors \
    import \
        InvalidPackage

_METADATA_EXPLICIT = ["name", "version", "summary", "url", "author",
        "author_email", "maintainer", "maintainer_email", "license", "description",
        "platforms", "install_requires", "build_requires", "download_url",
        "classifiers", "top_levels", "description_from_file", "keywords"]

_METADATA_FIELDS = _METADATA_EXPLICIT + ["version_major", "version_minor",
                                         "version_micro", "version_postdev"]

def _set_metadata(obj, name, version=None, summary=None, url=None,
        author=None, author_email=None, maintainer=None,
        maintainer_email=None, license=None, description=None,
        platforms=None, install_requires=None, build_requires=None,
        download_url=None, classifiers=None, top_levels=None,
        description_from_file=None, keywords=None):
    obj.name = name

    obj.version = version

    if version is None:
        obj.version_major = 0
        obj.version_minor = 0
        obj.version_micro = 0
        obj.version_postdev = ""
    else:
        if not is_valid_version(version):
            raise InvalidPackage("Invalid version: %r (suggested version: %s)" \
                                 % (version, suggest_normalized_version(version)))
        v = NormalizedVersion(version)
        obj.version_major = v.parts[0][0]
        obj.version_minor = v.parts[0][1]
        if len(v.parts[0]) > 2:
            obj.version_micro = v.parts[0][2]
        else:
            obj.version_micro = 0
        # FIXME: look at distutils version stuff more carefully
        obj.version_postdev = ""
        #if v.parts[1] != ('f',):
        #    raise InvalidPackage("Unsupported version: %r (prerelease part)" % (version,))
        #if v.parts[2] == ('f',):
        #    obj.version_postdev = ""
        #else:
        #    obj.version_postdev = "".join([str(i) for i in v.parts[2]])

    # FIXME: one should set metadata default elsewhere, and suggest good values
    # for developers
    obj.summary = summary or ""
    obj.url = url or ""
    obj.download_url = download_url or ""
    obj.author = author or ""
    obj.author_email = author_email or ""
    obj.maintainer = maintainer or ""
    obj.maintainer_email = maintainer_email or ""
    obj.license = license or ""
    obj.description = description or ""
    obj.description_from_file = description_from_file

    if not install_requires:
        obj.install_requires = []
    else:
        obj.install_requires = install_requires

    if not build_requires:
        obj.build_requires = []
    else:
        obj.build_requires = build_requires

    if not platforms:
        obj.platforms = []
    else:
        obj.platforms = platforms

    if not classifiers:
        obj.classifiers = []
    else:
        obj.classifiers = classifiers

    if not top_levels:
        obj.top_levels = []
    else:
        obj.top_levels = top_levels

    if not keywords:
        obj.keywords = []
    else:
        obj.keywords = keywords

    return obj

class PackageMetadata(object):
    metadata_attributes = _METADATA_FIELDS + ["fullname", "contact", "contact_email"]

    @classmethod
    def from_ipkg(cls, ipkg):
        return cls(**ipkg.meta)

    @classmethod
    def from_package(cls, pkg):
        kw = {}
        for k in _METADATA_EXPLICIT:
            if hasattr(pkg, k):
                kw[k] = getattr(pkg, k)
        return cls(**kw)

    def __init__(self, name, version=None, summary=None, url=None,
            author=None, author_email=None, maintainer=None,
            maintainer_email=None, license=None, description=None,
            platforms=None, install_requires=None, build_requires=None,
            download_url=None, classifiers=None, top_levels=None,
            description_from_file=None, keywords=None):
        # Package metadata
        _args = locals()
        kw = dict([(k, _args[k]) for k in _METADATA_FIELDS if k in _args])
        _set_metadata(self, **kw)

        # FIXME: not implemented yet
        self.provides = []
        self.obsoletes = []

    @property
    def fullname(self):
        return "%s-%s" % (self.name, self.version)

    @property
    def contact(self):
        return (self.maintainer or
                self.author or
                "UNKNOWN")

    @property
    def contact_email(self):
        return (self.maintainer_email or
                self.author_email or
                "UNKNOWN")



from bento.commands.core \
    import \
        Command, Option
from bento.commands.register \
    import \
        _read_pypirc
from bento.pypi.register_utils \
    import \
        DEFAULT_REPOSITORY, PyPIConfig
from bento.pypi.upload_utils \
    import \
        upload

import bento.errors

_SUPPORTED_DISTRIBUTIONS = {"source": "sdist", "egg": "bdist_egg"}

class UploadPyPI(Command):
    long_descr = """\
Purpose: register the package to pypi
Usage: bentomaker register [OPTIONS] distribution_file"""
    short_descr = "register packages to pypi."
    common_options = Command.common_options \
                        + [Option("-r", "--repository",
                                  help="Repository to use in .pypirc"),
                           Option("-u", "--username",
                                  help="Username to use for registration"),
                           Option("-p", "--password",
                                  help="Password to use for registration"),
                           Option("--repository-url",
                                  help="Repository URL to use for registration"),
                           Option("-t", "--distribution-type",
                                  help="Force distribution type (sdist, bdist_wininst, etc...)"),
                                  ]
    def run(self, context):
        o, a = context.get_parsed_arguments()
        if len(a) < 1:
            context.options_context.parser.print_usage()
            # FIXME
            raise NotImplementedError("expected file argument")
        else:
            filename = a[0]

        if o.repository and (o.username or o.password or o.repository_url):
            raise bento.errors.UsageException(
                    "Cannot specify repository and username/password/url at the same time")
        if not (o.repository or (o.username or o.password or o.repository_url)):
            # FIXME: why does distutils use DEFAULT_REPOSITORY (i.e. an url)
            # here ?
            config = _read_pypirc(DEFAULT_REPOSITORY)
        elif o.repository:
            config = _read_pypirc(o.repository)
        else:
            config = PyPIConfig(o.username, o.password, o.repository_url)

        if o.distribution_type is None:
            # FIXME
            raise NotImplementedError("automatic distribution type not yet implemented")
        if not o.distribution_type in _SUPPORTED_DISTRIBUTIONS:
            # FIXME
            raise NotImplementedError()

        upload_type = _SUPPORTED_DISTRIBUTIONS[o.distribution_type]
        upload(filename, upload_type, context.pkg, config=config)



import os.path as op
import re

import numpy.f2py
import numpy.distutils.misc_util

from waflib \
    import \
        Task
from waflib.TaskGen \
    import \
        extension, feature, before_method, after_method

from interface_gen \
    import \
        generate_interface


CGEN_TEMPLATE   = '%smodule'
FOBJECT_FILE    = 'fortranobject.c'
FHEADER_FILE    = 'fortranobject.h'
FWRAP_TEMPLATE  = '%s-f2pywrappers.f'

# This path is relative to build directory
F2PY_TEMP_DIR = '.f2py'

# Those regex are copied from build_src in numpy.distutils.command
F2PY_MODNAME_MATCH = re.compile(r'\s*python\s*module\s*(?P<name>[\w_]+)',
                                re.I).match
F2PY_UMODNAME_MATCH = re.compile(r'\s*python\s*module\s*(?P<name>[\w_]*?'\
                                     '__user__[\w_]*)',re.I).match
# End of copy
#F2PY_INCLUDE_MATCH = re.compile(r"^\s+\<include_file=(\S+)\>")
# Copied from crackfortran.py in f2py
F2PY_INCLUDE_MATCH = re.compile(r'\s*include\s*(\'|")(?P<name>[^\'"]*)(\'|")',re.I)

def is_pyf(node):
    return node.name.endswith(".pyf")

def include_pyf(node):
    includes = []
    for line in node.read().splitlines():
        m = F2PY_INCLUDE_MATCH.match(line)
        if m:
            includes.append(m.groupdict()["name"])
    return includes

def f2py_modulename(node):
    """This returns the name of the module from the pyf source file.

    source is expected to be one string, containing the whole source file
    code."""
    name = None
    for line in node.read().splitlines():
        m = F2PY_MODNAME_MATCH(line)
        if m:
            if F2PY_UMODNAME_MATCH(line): # skip *__user__* names
                continue
            name = m.group('name')
            break
    return name

def generate_fake_interface(name, node):
    """Generate a (fake) .pyf file from another pyf file (!)."""
    content = """\
python module %(name)s
    usercode void empty_module(void) {}
    interface
    subroutine empty_module()
    intent(c) empty_module
    end subroutine empty_module
    end interface
end python module%(name)s
"""
    node.write(content % {"name": name})

@feature('f2py')
@before_method('apply_incpaths')
def apply_f2py_includes(task_gen):
    includes = task_gen.env["INCLUDES"]
    d = op.dirname(numpy.f2py.__file__)
    includes.append(op.join(d, 'src'))
    includes.extend(numpy.distutils.misc_util.get_numpy_include_dirs())
    task_gen.env["INCLUDES"] = includes

@feature('f2py_fortran')
@before_method('process_source')
def add_fortran_tasks(task_gen):
    assert "f2py" in task_gen.features
    
    if hasattr(task_gen, "name"):
        module_name = task_gen.name
    else:
        raise ValueError("Please define a name for task_gen %r" % task_gen)
    if len(task_gen.source) < 1:
        raise ValueError("Gne ?")
    sources = task_gen.to_nodes(task_gen.source)
    module_node = sources[0].parent.find_or_declare("%smodule.c" % module_name)

    f2py_task = task_gen.create_task('f2py_fortran', sources, module_node)
    add_f2py_tasks(task_gen, f2py_task, module_name, module_node)
    f2py_task.env.F2PYFLAGS.extend(["--lower", "-m", module_name])

def fake_interface_gen_callback(task_gen, node):
    return _interface_gen_callback(task_gen, node, "f2py_fake_interface")

def interface_gen_callback(task_gen, node):
    return _interface_gen_callback(task_gen, node, "f2py_interface")

def _interface_gen_callback(task_gen, node, interface_task_name):
    if not hasattr(task_gen, "name"):
        module_name = f2py_modulename(node)
    else:
        module_name = task_gen.name

    intermediate_output = node.parent.find_or_declare("%s.pyf" % module_name)
    module_node = node.parent.find_or_declare("%smodule.c" % module_name)
    # Guard against overwriting existing source code by accident. Did I say I
    # hate find_or_declare ?
    assert intermediate_output.is_bld()
    assert module_node.is_bld()

    interface_task = task_gen.create_task(interface_task_name, node, intermediate_output)

    f2py_task = task_gen.create_task('f2py', intermediate_output, module_node)
    add_f2py_tasks(task_gen, f2py_task, module_name, module_node)

def add_f2py_tasks(task_gen, f2py_task, module_name, module_node):
    build_dir = module_node.parent.bldpath()
    # FIXME: ask waf ML how flags sharing and co is supposed to work
    f2py_task.env.F2PYFLAGS = task_gen.env.F2PYFLAGS[:]
    f2py_task.env.F2PYFLAGS.extend(["--build-dir", build_dir])
    task_gen.source += f2py_task.outputs

    fobject_node = module_node.parent.find_or_declare("%s-fortranobject.c" % module_name)
    fobject_task = task_gen.create_task("f2py_fortran_object", [], fobject_node)
    task_gen.source += fobject_task.outputs

    fwrapper_node = module_node.parent.find_or_declare(FWRAP_TEMPLATE % module_name)
    fwrapper_task = task_gen.create_task("f2py_fwrapper", [], fwrapper_node)
    task_gen.source += fwrapper_task.outputs

@extension('.pyf')
def add_f2py_files(task_gen, node):
    ext = '.c'

    if not is_pyf(node):
        raise ValueError("Gne ?")

    if not hasattr(task_gen, "name"):
        module_name = f2py_modulename(node)
    else:
        module_name = task_gen.name
    module_node = node.parent.find_or_declare("%smodule.c" % module_name)

    f2py_task = task_gen.create_task('f2py', node, module_node)
    add_f2py_tasks(task_gen, f2py_task, module_name, module_node)

@extension('.ipyf')
def add_f2py_files(task_gen, node):
    ext = '.pyf'

    if not hasattr(task_gen, "name"):
        module_name = f2py_modulename(node)
    else:
        module_name = task_gen.name
    intermediate_output = node.parent.find_or_declare("%s.pyf" % module_name)
    assert intermediate_output.is_bld()

    if "f2py_interface" in task_gen.features:
        interface_task = task_gen.create_task("f2py_interface", node, intermediate_output)
    elif "f2py_fake_interface" in task_gen.features:
        interface_task = task_gen.create_task("f2py_fake_interface", node, intermediate_output)
    else:
        raise ValueError("You need to use f2py_interface or f2py_fake_interface for .ipyf !")
    task_gen.source += interface_task.outputs
    #module_node = node.parent.find_or_declare("%smodule.c" % module_name)

    #f2py_task = task_gen.create_task('f2py', node, module_node)
    #add_f2py_tasks(task_gen, f2py_task, module_name, module_node)

class _f2py_interface(Task.Task):
    pass

class f2py_fake_interface(_f2py_interface):
    def run(self):
        node = self.inputs[0]
        output = self.outputs[0]
        module_name = f2py_modulename(node)
        generate_fake_interface(module_name, output)
        return 0

class f2py_interface(_f2py_interface):
    def run(self):
        node = self.inputs[0]
        output = self.outputs[0]
        module_name = f2py_modulename(node)
        generate_interface(module_name, node.abspath(), output.abspath())
        return 0

    def scan(self):
        found = []
        missing = []

        node = self.inputs[0]
        if not hasattr(self, "name"):
            module_name = f2py_modulename(node)
        else:
            module_name = self.name

        includes = include_pyf(node)
        if includes:
            real_pyf = node.parent.find_or_declare("%s.pyf" % module_name)
            # Guard against overwriting existing source code by accident. Did I
            # say I hate find_or_declare ?
            assert real_pyf.is_bld()

            for inc in includes:
                x = node.parent.find_resource(inc)
                if x:
                    found.append(x)
                else:
                    missing.append(x)
        user_routines = node.parent.find_resource("%s_user_routines.pyf" % module_name)
        if user_routines:
            found.append(user_routines)
        return (found, missing)

class f2py_fortran_object(Task.Task):
    def run(self):
        node = self.generator.bld.bldnode.find_node(op.join(F2PY_TEMP_DIR, FOBJECT_FILE))
        output = self.outputs[0]
        output.write(node.read())
        return 0

class f2py_fwrapper(Task.Task):
    after = ["f2py"]
    def run(self):
        output = self.outputs[0]
        if not op.exists(output.abspath()):
            output.write("")
        return 0

# TODO: f2py from .pyf or from .f should be different tasks.
class _f2py_task(Task.Task):
    run_str = '${F2PY} ${F2PYFLAGS} ${SRC}'
    color   = 'CYAN'

class f2py(_f2py_task):
    ext_in = [".pyf"]
    ext_out = [".h", ".f", ".c"]

class f2py_fortran(_f2py_task):
    ext_in = [".pyf"]
    ext_out = [".h", ".f", ".c"]

def configure(conf):
    if not conf.env.CC and not conf.env.CXX:
        conf.fatal('Load a C/C++ compiler first')
    if not conf.env.PYTHON:
        conf.fatal('Load the python tool first!')
    conf.find_program('f2py', var='F2PY')
    # FIXME: this has nothing to do here
    conf.env.F2PYFLAGS = ["--quiet"]

    f2py_tempdir = conf.bldnode.make_node(F2PY_TEMP_DIR)
    f2py_tempdir.mkdir()

    fobject = f2py_tempdir.make_node(FOBJECT_FILE)

    d = op.dirname(numpy.f2py.__file__)
    source_c = op.join(d, 'src', FOBJECT_FILE)
    fobject.write(open(source_c).read())



import os
import os.path as op
import tempfile
import shutil

from bento.compat.api.moves \
    import \
        unittest
from bento.distutils.dist \
    import \
        BentoDistribution
from bento.distutils.commands.install \
    import \
        install
        
class DistutilsCommandTestBase(unittest.TestCase):
    def setUp(self):
        self.old_cwd = os.getcwd()
        self.new_cwd = tempfile.mkdtemp()
        try:
            os.chdir(self.new_cwd)
        except:
            os.chdir(self.old_cwd)
            shutil.rmtree(self.new_cwd)
            raise

    def tearDown(self):
        os.chdir(self.old_cwd)
        shutil.rmtree(self.new_cwd)



import os
import sys
import shutil
import subprocess

from cStringIO \
    import \
        StringIO

import lib2to3.main

from yaku.errors \
    import \
        TaskRunFailure
from yaku.task \
    import \
        task_factory
from yaku.task_manager \
    import \
        TaskGen, extension
from yaku.pprint \
    import \
        pprint

import yaku.tools

def convert_func(self):
    if not len(self.inputs) == 1:
        raise ValueError("convert_func needs exactly one input")
    source, target = self.inputs[0], self.outputs[0]

    pprint('GREEN', "%-16s%s" % (self.name.upper(),
           " ".join([s.srcpath() for s in self.inputs])))
    cmd = ["2to3", "-w", "--no-diffs", "-n", source.abspath()]
    st = subprocess.call(cmd, stdout=subprocess.PIPE, stderr=subprocess.PIPE)
    if st != 0:
        pprint('RED', "FAILED %-16s%s" % (self.name.upper(),
               " ".join([s.srcpath() for s in self.inputs])))
        raise TaskRunFailure(cmd)
    target.write(source.read())

def copy_func(self):
    source, target = self.inputs[0], self.outputs[0]
    pprint('YELLOW', "%-16s%s" % (self.name.upper(),
           " ".join([s.srcpath() for s in self.inputs])))
    target.write(source.read())

class Py3kConverterBuilder(yaku.tools.Builder):
    def __init__(self, ctx):
        super(Py3kConverterBuilder, self).__init__(ctx)

    def _process_exclude(self, env):
        if "2TO3_EXCLUDE_LIST" in env:
            excludes = env["2TO3_EXCLUDE_LIST"]
        else:
            excludes = []
        dirs = []
        fs = []
        for e in excludes:
            n = self.ctx.src_root.find_node(e)
            if os.path.isdir(n.abspath()):
                dirs.append(n)
            else:
                fs.append(n)

        def _exclude(n):
            for d in dirs:
                if n.is_child_of(d):
                    return True
            if n in fs:
                return True
            return False
        return _exclude

    def convert(self, name, sources, env=None):
        # Basic principle: we first copy the whole tree (defined by the sources
        # list) into a temporary directory, which is then used for the
        # convertion. Notes:
        # - the whole tree needs to be copied before any 2to3 execution as 2to3
        # depends on the tree structure (e.g. for local vs absolute imports)
        # - because 2to3 can only modify files in place, we need to copy things
        # twice to avoid re-applying 2to3 several times (2to3 is not
        # idem-potent).
        # - the exclude process is particularly ugly...
        env = yaku.tools._merge_env(self.env, env)

        flter = self._process_exclude(env)
        self.env["__2TO3_FILTER"] = flter

        files = [self.ctx.src_root.find_resource(f) for f in sources]

        convert_tf = task_factory("2to3")
        copy_tf = task_factory("2to3_prepare")
        convert_tf.before.append(copy_tf.__name__)

        py3k_tmp = self.ctx.bld_root.declare("_py3k_tmp")
        py3k_top = self.ctx.bld_root.declare("py3k")
        tasks = []
        for f in files:
            target = py3k_tmp.declare(f.srcpath())
            task = copy_tf(inputs=[f], outputs=[target])
            task.func = copy_func
            task.env_vars = {}
            task.env = env
            tasks.append(task)

            if f.name.endswith(".py") and not flter(f):
                source = target
                target = py3k_top.declare(source.path_from(py3k_tmp))
                task = convert_tf(inputs=[source], outputs=[target])
                task.func = convert_func
                task.env_vars = {}
                task.env = env
                tasks.append(task)
            else:
                source = f
                target = py3k_top.declare(source.srcpath())
                task = copy_tf(inputs=[source], outputs=[target])
                task.func = copy_func
                task.env_vars = {}
                task.env = env
                tasks.append(task)

        self.ctx.tasks.extend(tasks)

        outputs = []
        for t in tasks:
            outputs.extend(t.outputs)
        return outputs

def get_builder(ctx):
    return Py3kConverterBuilder(ctx)



import sys
import os
import time

from distutils.util \
    import \
        get_platform
from distutils.sysconfig \
    import \
        get_python_version

from bento._config \
    import \
        WININST_DIR
from bento.core \
    import \
        PackageMetadata
import bento

import six

def wininst_filename(meta, pyver=None):
    if not pyver:
        pyver = ".".join([str(i) for i in sys.version_info[:2]])
    return "%s-py%s.win32.exe" % (meta.fullname, pyver)

# Stolen from distutils.commands.bdist_wininst
# FIXME: improve this code, in particular integration with
# InstalledPkgDescription

# FIXME: deal with this correctly, in particular MSVC - most likely we will
# need to hardcode things depending on python versions
def get_exe_bytes (target_version=None, plat_name=None):
    if target_version is None:
        target_version = ""
    if plat_name is None:
        plat_name = get_platform()
    from distutils.msvccompiler import get_build_version
    # If a target-version other than the current version has been
    # specified, then using the MSVC version from *this* build is no good.
    # Without actually finding and executing the target version and parsing
    # its sys.version, we just hard-code our knowledge of old versions.
    # NOTE: Possible alternative is to allow "--target-version" to
    # specify a Python executable rather than a simple version string.
    # We can then execute this program to obtain any info we need, such
    # as the real sys.version string for the build.
    cur_version = get_python_version()
    if target_version and target_version != cur_version:
        raise NotImplementedError("target_version handling not implemented yet.")
        # If the target version is *later* than us, then we assume they
        # use what we use
        # string compares seem wrong, but are what sysconfig.py itself uses
        if self.target_version > cur_version:
            bv = get_build_version()
        else:
            if self.target_version < "2.4":
                bv = 6.0
            else:
                bv = 7.1
    else:
        # for current version - use authoritative check.
        bv = get_build_version()

    # wininst-x.y.exe directory
    # XXX: put those somewhere else, and per-python version preferably
    directory = WININST_DIR
    # we must use a wininst-x.y.exe built with the same C compiler
    # used for python.  XXX What about mingw, borland, and so on?

    # if plat_name starts with "win" but is not "win32"
    # we want to strip "win" and leave the rest (e.g. -amd64)
    # for all other cases, we don't want any suffix
    if plat_name != 'win32' and plat_name[:3] == 'win':
        sfix = plat_name[3:]
    else:
        sfix = ''

    filename = os.path.join(directory, "wininst-%.1f%s.exe" % (bv, sfix))
    return open(filename, "rb").read()

def create_exe(ipkg, arcname, installer_name, bitmap=None, dist_dir="bento"):
    import struct

    if not os.path.exists(dist_dir):
        os.makedirs(dist_dir)

    cfgdata = get_inidata(ipkg)

    if bitmap:
        bitmapdata = open(bitmap, "rb").read()
        bitmaplen = len(bitmapdata)
    else:
        bitmaplen = 0

    fid = open(installer_name, "wb")
    fid.write(get_exe_bytes())
    if bitmap:
        fid.write(bitmapdata)

    # Convert cfgdata from unicode to ascii, mbcs encoded
    cfgdata = cfgdata.encode("mbcs") + six.b("\0")
    fid.write(cfgdata)

    # The 'magic number' 0x1234567B is used to make sure that the
    # binary layout of 'cfgdata' is what the wininst.exe binary
    # expects.  If the layout changes, increment that number, make
    # the corresponding changes to the wininst.exe sources, and
    # recompile them.
    header = struct.pack("<iii",
                         0x1234567B,       # tag
                         len(cfgdata),     # length
                         bitmaplen,        # number of bytes in bitmap
                         )
    fid.write(header)
    fid.write(open(arcname, "rb").read())

def get_inidata(ipkg):
    # Return data describing the installation.
    meta = PackageMetadata.from_ipkg(ipkg)

    # Write the [metadata] section.
    lines = []
    lines.append("[metadata]")

    # 'info' will be displayed in the installer's dialog box,
    # describing the items to be installed.
    info = meta.description + '\n'

    # Escape newline characters
    def escape(s):
        return s.replace("\n", "\\n")

    for name in ["author", "author_email", "summary", "maintainer",
                 "maintainer_email", "name", "url", "version"]:
        data = getattr(meta, name)
        if name == "summary":
            name = "description"
        if data:
            info = info + ("\n    %s: %s" % \
                           (name.capitalize(), escape(data)))
            lines.append("%s=%s" % (name, escape(data)))

    # The [setup] section contains entries controlling
    # the installer runtime.
    lines.append("\n[Setup]")
    # FIXME: handle install scripts
    #if self.install_script:
    #    lines.append("install_script=%s" % self.install_script)
    lines.append("info=%s" % escape(info))
    # FIXME: handle this correctly
    lines.append("target_compile=1")
    lines.append("target_optimize=1")
    #if self.target_version:
    #    lines.append("target_version=%s" % self.target_version)
    #if self.user_access_control:
    #    lines.append("user_access_control=%s" % self.user_access_control)

    title = meta.fullname
    lines.append("title=%s" % escape(title))
    build_info = "Built %s with bento-%s" % \
                 (time.ctime(time.time()), bento.__version__)
    lines.append("build_info=%s" % build_info)
    return "\n".join(lines)



import os
import os.path as op
import shutil
import tempfile

from bento.compat.api.moves \
    import \
        unittest

from bento.core.node \
    import \
        create_root_with_source_tree
from bento.core.testing \
    import \
        create_fake_package_from_bento_infos
from bento.commands.command_contexts \
    import \
        ConfigureContext
from bento.commands.hooks \
    import \
        create_hook_module, find_pre_hooks, find_post_hooks

from bento.commands.tests.utils \
    import \
        prepare_configure

class TestHooks(unittest.TestCase):
    def setUp(self):
        self.old_cwd = os.getcwd()
        self.new_cwd = tempfile.mkdtemp()
        try:
            os.chdir(self.new_cwd)
            try:
                root = create_root_with_source_tree(self.new_cwd,
                        op.join(self.new_cwd, "build"))
                self.top_node = root._ctx.srcnode
                self.build_node = root._ctx.bldnode
            except:
                os.chdir(self.old_cwd)
                raise
        except:
            shutil.rmtree(self.new_cwd)
            raise

    def tearDown(self):
        os.chdir(self.old_cwd)
        shutil.rmtree(self.new_cwd)

    def test_simple(self):
        bscript = """\
from bento.commands import hooks

PRE = False
POST = False

@hooks.pre_configure
def pre_configure(ctx):
    global PRE
    PRE = True

@hooks.post_configure
def post_configure(ctx):
    global POST
    POST = True
"""

        bento = """\
Name: foo
Version: 1.0

HookFile: bscript
"""

        create_fake_package_from_bento_infos(self.top_node, {"bento.info": bento},
                {"bscript": bscript})

        conf, configure = prepare_configure(self.top_node, bento,
                ConfigureContext)
        bscript = self.top_node.search("bscript")
        m = create_hook_module(bscript.abspath())
        self.assertEqual(len(find_pre_hooks([m], "configure")), 1)
        self.assertEqual(len(find_post_hooks([m], "configure")), 1)



import os
import tempfile

from bento.commands.build_distutils \
    import \
        DistutilsBuilder
from bento.compat.api.moves \
    import \
        unittest
from bento.core.node \
    import \
        create_base_nodes
from bento.core.testing \
    import \
        require_c_compiler, DUMMY_C, DUMMY_CLIB
from bento.core.pkg_objects \
    import \
        Extension, CompiledLibrary

class TestDistutilsBuilder(unittest.TestCase):
    def setUp(self):
        self.old_cwd = os.getcwd()
        new_cwd = tempfile.mkdtemp()
        os.chdir(new_cwd)
        try:
            self.top_node, self.build_node, self.run_node = create_base_nodes()
        except:
            os.chdir(self.old_cwd)
            raise

    def tearDown(self):
        os.chdir(self.old_cwd)

    #@require_c_compiler("distutils")
    def test_extension_simple(self):
        self.top_node.make_node("foo.c").write(DUMMY_C % {"name": "foo"})

        builder = DistutilsBuilder()
        extension = Extension("foo", ["foo.c"])
        builder.build_extension(extension)

    @require_c_compiler("distutils")
    def test_multiple_extensions(self):
        self.top_node.make_node("foo.c").write(DUMMY_C % {"name": "foo"})
        self.top_node.make_node("bar.c").write(DUMMY_C % {"name": "bar"})

        builder = DistutilsBuilder()

        extensions = [Extension("bar", ["bar.c"]), Extension("foo", ["foo.c"])]
        for extension in extensions:
            builder.build_extension(extension)

    @require_c_compiler("distutils")
    def test_clib_simple(self):
        self.top_node.make_node("foo.c").write(DUMMY_CLIB % {"name": "foo"})

        builder = DistutilsBuilder()
        extension = CompiledLibrary("foo", ["foo.c"])
        builder.build_compiled_library(extension)

    @require_c_compiler("distutils")
    def test_multiple_clibs(self):
        self.top_node.make_node("foo.c").write(DUMMY_CLIB % {"name": "foo"})
        self.top_node.make_node("bar.c").write(DUMMY_CLIB % {"name": "bar"})

        builder = DistutilsBuilder()

        extensions = [CompiledLibrary("bar", ["bar.c"]), CompiledLibrary("foo", ["foo.c"])]
        for extension in extensions:
            builder.build_compiled_library(extension)



import os
import os.path as op

from bento.distutils.utils \
    import \
        _is_setuptools_activated
if _is_setuptools_activated():
    from setuptools.command.egg_info \
        import \
            egg_info as old_egg_info
else:
    raise ValueError("You cannot use egg_info without setuptools enabled first")

from bento._config \
    import \
        IPKG_PATH
from bento.installed_package_description \
    import \
        InstalledPkgDescription
from bento.commands.egg_utils \
    import \
        EggInfo

class egg_info(old_egg_info):
    def run(self):
        self.run_command("build")
        dist = self.distribution

        n = dist.build_node.make_node(IPKG_PATH)
        ipkg = InstalledPkgDescription.from_file(n.abspath())

        egg_info = EggInfo.from_ipkg(ipkg, dist.build_node)

        egg_info_dir = op.join(self.egg_base, "%s.egg-info" % dist.pkg.name)
        try:
            os.makedirs(egg_info_dir)
        except OSError, e:
            if e.errno != 17:
                raise
        for filename, cnt in egg_info.iter_meta(dist.build_node):
            filename = op.join(egg_info_dir, filename)
            fid = open(filename, "w")
            try:
                fid.write(cnt)
            finally:
                fid.close()



import os.path as op

from six.moves \
    import \
        StringIO

from bento.backends.yaku_backend \
    import \
        ConfigureYakuContext, BuildYakuContext, YakuBackend
from bento.commands.build \
    import \
        BuildCommand
from bento.commands.configure \
    import \
        ConfigureCommand
from bento.commands.contexts \
    import \
        GlobalContext
from bento.commands.options \
    import \
        OptionsContext
from bento.core \
    import \
        PackageDescription, PackageOptions
from bento.core.testing \
    import \
        create_fake_package_from_bento_info

def prepare_configure(run_node, bento_info, context_klass=ConfigureYakuContext, cmd_argv=None):
    if cmd_argv is None:
        cmd_argv = []
    return _prepare_command(run_node, bento_info, ConfigureCommand, context_klass, cmd_argv)

def prepare_build(run_node, bento_info, context_klass=BuildYakuContext, cmd_argv=None):
    if cmd_argv is None:
        cmd_argv = []
    return _prepare_command(run_node, bento_info, BuildCommand, context_klass, cmd_argv)

def _prepare_command(run_node, bento_info, cmd_klass, context_klass, cmd_argv):
    top_node = run_node._ctx.srcnode
    top_node.make_node("bento.info").safe_write(bento_info)

    package = PackageDescription.from_string(bento_info)
    package_options = PackageOptions.from_string(bento_info)

    cmd = cmd_klass()
    options_context = OptionsContext.from_command(cmd)

    cmd.register_options(options_context, package_options)

    global_context = GlobalContext(None)
    global_context.register_package_options(package_options)

    context = context_klass(global_context, cmd_argv, options_context, package, run_node)
    return context, cmd

def create_global_context(package, package_options, backend=None):
    if backend is None:
        backend = YakuBackend()

    global_context = GlobalContext(None)
    global_context.register_package_options(package_options)
    if backend:
        global_context.backend = backend

    build = BuildCommand()
    configure = ConfigureCommand()

    commands = (("configure", configure), ("build", build))

    for cmd_name, cmd in commands:
        global_context.register_command(cmd_name, cmd)
        options_context = OptionsContext.from_command(cmd)
        global_context.register_options_context(cmd_name, options_context)

    global_context.backend.register_command_contexts(global_context)
    global_context.backend.register_options_contexts(global_context)

    return global_context

def prepare_package(top_node, bento_info, backend=None):
    package = PackageDescription.from_string(bento_info)
    package_options = PackageOptions.from_string(bento_info)

    create_fake_package_from_bento_info(top_node, bento_info)
    top_node.make_node("bento.info").safe_write(bento_info)

    return create_global_context(package, package_options, backend)

def prepare_command(global_context, cmd_name, cmd_argv, package, run_node):
    if cmd_argv is None:
        cmd_argv = []

    def _create_context(cmd_name):
        context_klass = global_context.retrieve_command_context(cmd_name)
        options_context = global_context.retrieve_options_context(cmd_name)
        context = context_klass(global_context, cmd_argv, options_context, package, run_node)
        return context

    return _create_context(cmd_name), global_context.retrieve_command(cmd_name)

# Super ugly stuff to make waf and nose happy: nose happily override
# sys.stdout/sys.stderr, and waf expects real files (with encoding and co). We
# fake it until waf is happy
class EncodedStringIO(object):
    def __init__(self):
        self._data = StringIO()
        self.encoding = "ascii"

    def read(self):
        return self._data.read()

    def write(self, data):
        return self._data.write(data)

    def flush(self):
        self._data.flush()

def comparable_installed_sections(sections):
    # Hack to compare compiled sections without having to hardcode the exact,
    # platform-dependent name
    class _InstalledSectionProxy(object):
        def __init__(self, installed_section):
            self.installed_section = installed_section

        def __eq__(self, other):
            _self = self.installed_section
            _other = other.installed_section
            def are_files_comparables():
                if len(_self.files) != len(_other.files):
                    return False
                else:
                    for i in range(len(_self.files)):
                        for j in range(2):
                            if op.dirname(_self.files[i][j]) != \
                                    op.dirname(_other.files[i][j]):
                                return False
                    return True

            return _self.name == _other.name and \
                _self.source_dir == _other.source_dir and \
                _self.target_dir == _other.target_dir and \
                are_files_comparables()

        def __repr__(self):
            return self.installed_section.__repr__()

    def proxy_categories(sections):
        for category in sections:
            for section_name, section in sections[category].items():
                sections[category ][section_name] = _InstalledSectionProxy(section)
        return sections

    return proxy_categories(sections)



import os
import shutil
import sys
import tempfile
import ntpath
import posixpath

import os.path as op

from six \
    import \
        BytesIO

from subprocess \
    import \
        PIPE, STDOUT, Popen

from bento.utils.utils \
    import \
        pprint

distutils_code = """\
import sys
import distutils.core

from distutils.core import setup as _setup

DIST = None

def new_setup(**kw):
    global DIST
    dist = _setup(**kw)
    DIST = dist
    return dist

filename = '%(filename)s'

globals = {}
globals["__name__"] = "__main__"
globals["__file__"] = filename

sys.argv = [filename, "config"]
distutils.core.setup = new_setup
sys.path.insert(0, '%(odir)s')

fp = open(filename, "rt")
try:
    exec(fp.read(), globals)
finally:
    fp.close()

if DIST is None:
    sys.exit(-2)

if DIST.__module__ == "distutils.dist":
    sys.exit(0)
else:
    sys.exit(-1)
"""

setuptools_code = """\
import sys
import setuptools
import distutils.core

from setuptools import setup as _setup

DIST = None

def new_setup(**kw):
    global DIST
    dist = _setup(**kw)
    DIST = dist
    return dist

filename = '%(filename)s'

globals = {}
globals["__name__"] = "__main__"
globals["__file__"] = filename

sys.argv = [filename, "-q", "--help"]
setuptools.setup = new_setup
distutils.core.setup = new_setup
sys.path.insert(0, '%(odir)s')
fp = open(filename, "rt")
try:
    exec(fp.read(), globals)
finally:
    fp.close()


if DIST is None:
    sys.exit(-2)

if DIST.__module__ == "setuptools.dist":
    sys.exit(0)
else:
    sys.exit(-1)
"""

numpy_code = """\
import sys
try:
    import numpy.distutils.core
    from numpy.distutils.core import setup as _setup

    DIST = None

    def new_setup(**kw):
        global DIST
        dist = _setup(**kw)
        DIST = dist
        return dist

    filename = '%(filename)s'

    globals = {}
    globals["__name__"] = "__main__"
    globals["__file__"] = filename

    sys.argv = [filename, "-q", "--name"]
    numpy.distutils.core.setup = new_setup
    sys.path.insert(0, '%(odir)s')

    fp = open(filename, "rt")
    try:
        exec(fp.read(), globals)
    finally:
        fp.close()

    if DIST is None:
        sys.exit(-2)

    if DIST.__module__ == "numpy.distutils.numpy_distribution":
        sys.exit(0)
    else:
        sys.exit(-1)
except ImportError:
    sys.exit(-1)
"""

setuptools_numpy_code = """\
import sys
try:
    import setuptools
    import numpy.distutils.core
    from numpy.distutils.core import setup as _setup

    DIST = None

    def new_setup(**kw):
        global DIST
        dist = _setup(**kw)
        DIST = dist
        return dist

    filename = '%(filename)s'

    globals = {}
    globals["__name__"] = "__main__"
    globals["__file__"] = filename

    sys.argv = [filename, "-q", "--name"]
    numpy.distutils.core.setup = new_setup
    sys.path.insert(0, '%(odir)s')
    fp = open(filename, "rt")
    try:
        exec(fp, globals)
    finally:
        fp.close()


    if DIST is None:
        sys.exit(-2)

    if DIST.__module__ == "numpy.distutils.numpy_distribution" and \\
        "setuptools" in sys.modules:
        sys.exit(0)
    else:
        sys.exit(-1)
except ImportError:
    sys.exit(-1)
"""

can_run_code = """\
import sys

filename = '%(filename)s'

globals = {}
globals["__name__"] = "__main__"
globals["__file__"] = filename

sys.argv = [filename, "-q", "--name"]
sys.path.insert(0, '%(odir)s')

fp = open(filename, "rt")
try:
    exec(fp.read(), globals)
finally:
    fp.close()
"""

def logged_run(cmd, buffer):
    """Return exit code."""
    pid = Popen(cmd, stdout=PIPE, stderr=STDOUT)
    pid.wait()

    buffer.write(pid.stdout.read())
    return pid.returncode
    
def _test(code, setup_py, show_output, log):
    d = tempfile.mkdtemp()
    try:
        filename = op.join(d, "setup.py")

        cmd = [sys.executable, filename]
        log.write(" | Running %s, content below\n" % " ".join(cmd))
        log.writelines([" | | %s\n" % line for line in code.splitlines()])

        fp = open(filename, "wt")
        try:
            fp.write(code)
        finally:
            fp.close()

        buf = BytesIO()
        st = logged_run(cmd, buf)

        # FIXME: handle this correctly
        log.write(" | return of the command is %d and output is\n" % st)
        val = buf.getvalue()
        if val:
            log.writelines([" | | %s\n" % line for line in val.splitlines()])
            log.write("\n")
        else:
            log.write(" | (None)\n")
        log.write("\n")
        return st == 0
    finally:
        shutil.rmtree(d)

def test_distutils(setup_py, show_output, log):
    odir = os.path.dirname(os.path.abspath(setup_py))
    log.write("bentomaker: convert\n")
    log.write(" -> testing straight distutils\n")
    return _test(distutils_code % {"filename": setup_py, "odir": odir}, setup_py,
                 show_output, log)

def test_setuptools(setup_py, show_output, log):
    odir = os.path.dirname(os.path.abspath(setup_py))
    log.write("bentomaker: convert\n")
    log.write(" -> testing setuptools\n")
    return _test(setuptools_code % {"filename": setup_py, "odir": odir}, setup_py,
                 show_output, log)

def test_numpy(setup_py, show_output, log):
    odir = os.path.dirname(os.path.abspath(setup_py))
    log.write("bentomaker: convert\n")
    log.write(" -> testing straight numpy.distutils\n")
    return _test(numpy_code % {"filename": setup_py, "odir": odir}, setup_py,
                 show_output, log)

def test_setuptools_numpy(setup_py, show_output, log):
    odir = os.path.dirname(os.path.abspath(setup_py))
    log.write("bentomaker: convert\n")
    log.write(" -> testing numpy.distutils monkey-patched by setuptools\n")
    return _test(setuptools_numpy_code % {"filename": setup_py, "odir": odir}, setup_py,
                 show_output, log)

def test_can_run(setup_py, show_output, log):
    odir = os.path.dirname(os.path.abspath(setup_py))
    log.write("bentomaker: convert\n")
    log.write(" -> testing whether setup.py can be executed without errors\n")
    return _test(can_run_code % {"filename": setup_py, "odir": odir}, setup_py,
                 show_output, log)

def whole_test(setup_py, verbose, log):
    if verbose:
        show_output = True
    else:
        show_output = False

    if not test_can_run(setup_py, show_output, log):
        pass
    if verbose:
        pprint("YELLOW", "----------------- Testing distutils ------------------")
    use_distutils = test_distutils(setup_py, show_output, log)
    if verbose:
        pprint("YELLOW", "----------------- Testing setuptools -----------------")
    use_setuptools = test_setuptools(setup_py, show_output, log)
    if verbose:
        pprint("YELLOW", "------------ Testing numpy.distutils -----------------")
    use_numpy = test_numpy(setup_py, show_output, log)
    if verbose:
        pprint("YELLOW", "--- Testing numpy.distutils patched by setuptools ----")
    use_setuptools_numpy = test_setuptools_numpy(setup_py, show_output, log)
    if verbose:
        print("Is distutils ? %d" % use_distutils)
        print("Is setuptools ? %d" % use_setuptools)
        print("Is numpy distutils ? %d" % use_numpy)
        print("Is setuptools numpy ? %d" % use_setuptools_numpy)

    if use_distutils and not (use_setuptools or use_numpy or use_setuptools_numpy):
        return "distutils"
    elif use_setuptools  and not (use_numpy or use_setuptools_numpy):
        return "setuptools"
    elif use_numpy  and not use_setuptools_numpy:
        return "numpy.distutils"
    elif use_setuptools_numpy:
        return "setuptools + numpy.distutils converter"
    else:
        return "Unsupported converter"

def canonalize_path(path):
    """Convert a win32 path to unix path."""
    head, tail = ntpath.split(path)
    lst = [tail]
    while head and tail:
        head, tail = ntpath.split(head)
        lst.insert(0, tail)
    lst.insert(0, head)

    return posixpath.join(*lst)
